{   Unit cyImageEn

    Description:
    Unit with functions to use with ImageEn5.X/ImageEn6.X components.

    $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    $   Accept any PAYPAL DONATION $$$  
    $      to: mauricio_box@yahoo.com      
    

    * ***** BEGIN LICENSE BLOCK *****
    *
    * Version: MPL 1.1
    *
    * The contents of this file are subject to the Mozilla Public License Version
    * 1.1 (the "License"); you may not use this file except in compliance with the
    * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
    *
    * Software distributed under the License is distributed on an "AS IS" basis,
    * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
    * the specific language governing rights and limitations under the License.
    *
    * The Initial Developer of the Original Code is Mauricio
    * (https://sourceforge.net/projects/tcycomponents/).
    *
    * Donations: see Donation section on Description.txt
    *
    * Alternatively, the contents of this file may be used under the terms of
    * either the GNU General Public License Version 2 or later (the "GPL"), or the
    * GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
    * case the provisions of the GPL or the LGPL are applicable instead of those
    * above. If you wish to allow use of your version of this file only under the
    * terms of either the GPL or the LGPL, and not to allow others to use your
    * version of this file under the terms of the MPL, indicate your decision by
    * deleting the provisions above and replace them with the notice and other
    * provisions required by the LGPL or the GPL. If you do not delete the
    * provisions above, a recipient may use your version of this file under the
    * terms of any one of the MPL, the GPL or the LGPL.
    *
    * ***** END LICENSE BLOCK *****}

unit cyImageEn;

interface

uses Windows, Classes, SysUtils, imageenio, iemio, iewia, imscan, iexAcquire, iemview, imageenproc, hyiedefs;

// Twain functions :
function TWAIN_SilentAcquireFromFlatbed(Handler: TImageEnIO; AfterScanSheet: TNotifyEvent): Boolean;
function TWAIN_SilentAcquireFromFeeder(Handler: TImageEnMIO; Pages: Integer): Boolean;

// WIA functions:
function UseWIACommands_ForVistaAndUpper: boolean;
function WIA_Connected(aWIAParams: TIEWia): Boolean;        // Device connected?
function WIA_FeedPageReady(aWIAParams: TIEWia): Boolean;    // Feeder with sheets?
function WIA_DuplexSupported(aWIAParams: TIEWia): Boolean;  // Duplex feature?
function WIA_paperJam(aWIAParams: TIEWia): Boolean;         // Paper jam?
//function WIA_SilentAcquire(Handler: TImageEnIO; AfterScanSheet: TNotifyEvent): boolean;
function WIA_SilentAcquireFromFlatbed(Handler: TImageEnIO; AfterScanSheet: TNotifyEvent): boolean;
function WIA_SilentAcquireFromFeeder(Handler: TImageEnMIO; DuplexMode: Boolean; Pages: Integer): boolean;

// Other functions :
procedure ConvertTiff2Pdf(ImageEnMView: TImageEnMView; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0); overload;  // Diference between 80 and 90 is not big ...
procedure ConvertTiff2Pdf(FileTif, FilePdf: String; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0); overload;    // Diference between 80 and 90 is not big ...

procedure ConvertImage2Pdf(ImageEnMView: TImageEnMView; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0); overload;   // Diference between 80 and 90 is not big ...
procedure ConvertImage2Pdf(ImageFile, FilePdf: String; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0); overload;   // Diference between 80 and 90 is not big ...

procedure ConvertImage2Tiff(ImageEnMView: TImageEnMView; const TIFF_Compression: TIOTIFFCompression = ioTIFF_G4FAX; const ResampleDpis: Integer = 0); overload;
procedure ConvertImage2Tiff(ImageFile, FileTiff: String; const TIFF_Compression: TIOTIFFCompression = ioTIFF_G4FAX; const ResampleDpis: Integer = 0); overload;

function EncodeTiffToG4FAX(const SrcFileTIF, DestFileTIF: String; var Modified: Boolean; const BWLocalThresholdParam: Integer = 16): Boolean;

function SaveToTiff(ImageEnMView: TImageEnMView; const DestFileTiff: String): Boolean;
function SaveToPdf(ImageEnMView: TImageEnMView; const DestFilePdf: String): Boolean;

const
  TWAIN_CAP_XFERCOUNT = $0001; // number of pages (default is -1)

  // For Windows Vista and upper (from Microsoft SDK wiadef.h header file)
  WIA_IPS_PAGES = 3096;                 // Page number to scan from feeder
  WIA_IPS_PAGE_SIZE = 3097;             // Page definition
  WIA_IPS_PAGE_WIDTH = 3098;            // A4 =  8267 inches
  WIA_IPS_PAGE_HEIGHT = 3099;           // A4 = 11692 inches
  WIA_IPS_MAX_HORIZONTAL_SIZE = 6165;   // Maximum Horizontal Scan Size

  WIA_IPS_MAX_VERTICAL_SIZE = 6166;     // Maximum Vertical Scan Size
  WIA_IPS_MIN_HORIZONTAL_SIZE = 6167;   // Minimum Horizontal Scan Size
  WIA_IPS_MIN_VERTICAL_SIZE = 6168;     // Minimum Vertical Scan Size


implementation

// Twain functions :
// RHR - Does not work if pages are in the feeder!
function TWAIN_SilentAcquireFromFlatbed(Handler: TImageEnIO; AfterScanSheet: TNotifyEvent): Boolean;
begin
  Result := true;
  Handler.TWainParams.VisibleDialog := false;
  Handler.TWainParams.ProgressIndicators:= false;
  Handler.TWainParams.FeederEnabled := false;
  Handler.TWainParams.AutoFeed := false;
  Handler.TWainParams.AcceptedImages := -2;  // Default scanner value

  // Digitalize once :
  if Handler.Acquire then // ImageEn3    if Handler.Acquire(ieaTWain) then
    begin
      if Assigned(AfterScanSheet) then
        AfterScanSheet(Handler);
    end
    else
      Result := false;
end;

function TWAIN_SilentAcquireFromFeeder(Handler: TImageEnMIO; Pages: Integer): Boolean;
begin
  Handler.TWainParams.VisibleDialog := false;
  Handler.TWainParams.ProgressIndicators:= false;
  Handler.TWainParams.FeederEnabled := true;

{ ImageEn3
  Handler.TWainParams.AutoFeed := Pages <> 1;

  if pages > 0
  then imscan.FeederScanPages := Pages
  else imscan.FeederScanPages := -1;    }

  // No faz diferna ...    Handler.TWainParams.AutoScan := true;
  Handler.TWainParams.AutoFeed := Pages = 0;  // Scan all pages

  if pages > 0
  then Handler.TWainParams.AcceptedImages := Pages
  else Handler.TWainParams.AcceptedImages := -1;    // Multiple pages


  Result := Handler.Acquire;   // ImageEn3  Handler.Acquire(ieaTWain);
end;

// WIA functions :
function UseWIACommands_ForVistaAndUpper: boolean;
var
  VersionInfo: TOSVersionInfo;
begin
  Result := false;
  VersionInfo.dwOSVersionInfoSize:=SizeOf(VersionInfo);
  GetVersionEx(VersionInfo);

  if VersionInfo.dwPlatformId = VER_PLATFORM_WIN32_NT  // Win 2000, XP, Vista and 7 ...
  then Result := VersionInfo.dwMajorVersion >= 6;
end;

function WIA_Connected(aWIAParams: TIEWia): Boolean;
var
  ov: Variant;
begin
  RESULT := false;

  try
    ov := aWIAParams.GetItemProperty(WIA_DPA_CONNECT_STATUS, nil);

    if ov = WIA_DEVICE_CONNECTED then
      RESULT := true;
  except

  end;
end;

function WIA_FeedPageReady(aWIAParams: TIEWia): Boolean;
var
  ov: Variant;
  Valor: Integer;
begin
  RESULT := false;

  try
    ov := aWIAParams.GetDeviceProperty(WIA_DPS_DOCUMENT_HANDLING_STATUS);
    valor := ov;

    if ov <> 0 then
      if Valor and WIA_FEED_READY = Valor then
        RESULT := true;
  except

  end;
end;

function WIA_DuplexSupported(aWIAParams: TIEWia): Boolean;
var
  ov: Variant;
begin
  RESULT := false;

  try
    ov := aWIAParams.GetDeviceProperty(WIA_DETECT_DUP_AVAIL);

    if ov <> 0 then
      RESULT := true;
  except

  end;
end;

function WIA_paperJam(aWIAParams: TIEWia): Boolean;
var
  ov: Variant;
begin
  RESULT := false;

  try
    ov := aWIAParams.GetDeviceProperty(WIA_PAPER_JAM);

    if ov <> 0 then
      RESULT := true;
  except

  end;
end;

function WIA_SilentAcquireFromFlatbed(Handler: TImageEnIO; AfterScanSheet: TNotifyEvent): boolean;
var
  Flag: Integer;
begin
  Result := true;
  Handler.AcquireParams.VisibleDialog := false;
  Flag   := WIA_FLATBED;
  Handler.WIAParams.SetDeviceProperty(WIA_DPS_DOCUMENT_HANDLING_SELECT, Flag);

  if Handler.Acquire then // ImageEn3   if Handler.Acquire(ieaWIA) then
  begin
    if Assigned(AfterScanSheet) then
      AfterScanSheet(Handler);
  end
  else
    Result := false;
end;

function WIA_SilentAcquireFromFeeder(Handler: TImageEnMIO; DuplexMode: Boolean; Pages: Integer): boolean;
var Flag: Integer;
begin
  Handler.AcquireParams.VisibleDialog := false;

  if Pages < 0
  then Pages := 0; // Multipages ...

  if DuplexMode
  then Flag := WIA_FEEDER or WIA_DUPLEX
  else Flag := WIA_FEEDER;

  Handler.WIAParams.SetDeviceProperty(WIA_DPS_DOCUMENT_HANDLING_SELECT, Flag);

  if UseWIACommands_ForVistaAndUpper
  then Handler.WIAParams.SetDeviceProperty(WIA_IPS_PAGES, Pages)
  else Handler.WIAParams.SetDeviceProperty(WIA_DPS_PAGES, Pages);

  Result := Handler.Acquire; // ImageEn3  Result := Handler.Acquire(ieaWIA);
end;

procedure ConvertTiff2Pdf(ImageEnMView: TImageEnMView; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0);   // Diference between 80 and 90 is not big ...
var
  p: Integer;
begin
  for p := 1 to ImageEnMView.ImageCount do
  begin
    ImageEnMView.SelectedImage := p-1;

    if ImageEnMView.IEBitmap.Width <> 0 then    // 2016-07-29 Force to load IEBitmap reading any property !!!
    begin
      if ResampleDpis <> 0 then
        if ImageEnMView.MIO.Params[p-1].Dpi <> ResampleDpis then
          ImageEnMView.Proc.Resample( Round(ImageEnMView.MIO.Params[p-1].Width * ResampleDpis / ImageEnMView.MIO.Params[p-1].Dpi),    // New width
                                     -1,   // New height (Auto) ...
                                     rfFastLinear, True);

      if ImageEnMView.MIO.Params[p-1].BitsPerSample = 1 then
      begin
        ImageEnMView.MIO.Params[p-1].JPEG_Quality := JpegQuality;
        ImageEnMView.MIO.Params[p-1].PDF_Compression := ioPDF_G4FAX;
      end
      else begin
        ImageEnMView.MIO.Params[p-1].JPEG_Quality := ImageEnMView.MIO.Params[p-1].TIFF_JPEGQuality;
        ImageEnMView.MIO.Params[p-1].PDF_Compression := ioPDF_JPEG;
      end;
    end;

    ImageEnMView.MIO.Params[p-1].JPEG_Smooth := 100;

    (* 2016-07-29 ImageEnMView.MIO.Params[j].JPEG_Quality := JpegQuality;
    ImageEnMView.MIO.Params[j].PDF_Compression := ioPDF_JPEG;  *)
  end;
end;

procedure ConvertTiff2Pdf(FileTif, FilePdf: String; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0);   // Diference between 80 and 90 is not big ...
var
  ImageEnMView: TImageEnMView;
  p: Integer;
begin
  ImageEnMView := TImageEnMView.Create(Nil);

  (* 2016-07-29 Error Out of memory on large tif (16 MO) ...
  ImageEnMView.EnableImageCaching := True;
  ImageEnMView.ImageCacheUseDisk := False;          // !!! On ne peut pas utiliser ImageCacheUseDisk quand on utilise le LogonUser !!!
  ImageEnMView.MIO.LoadFromFileTIFF(FileTif); *)

  ImageEnMView.LoadFromFileOnDemand(FileTif, false);
  ConvertTiff2Pdf(ImageEnMView, JpegQuality, ResampleDpis);

  ImageEnMView.MIO.SaveToFilePDF(FilePdf);
  ImageEnMView.Free;
end;

procedure ConvertImage2Pdf(ImageEnMView: TImageEnMView; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0);   // Diference between 80 and 90 is not big ...
var
  p: Integer;
begin
  for p := 1 to ImageEnMView.ImageCount do
  begin
    ImageEnMView.SelectedImage := p-1;

    if ImageEnMView.IEBitmap.Width <> 0 then    // 2016-07-29 Force to load IEBitmap reading any property !!!
    begin
      if ResampleDpis <> 0 then
        if ImageEnMView.MIO.Params[p-1].Dpi <> ResampleDpis then
          ImageEnMView.Proc.Resample( Round(ImageEnMView.MIO.Params[p-1].Width * ResampleDpis / ImageEnMView.MIO.Params[p-1].Dpi),    // New width
                                     -1,   // New height (Auto) ...
                                     rfFastLinear, True);

      ImageEnMView.MIO.Params[p-1].JPEG_Quality := JpegQuality;
      ImageEnMView.MIO.Params[p-1].PDF_Compression := ioPDF_JPEG;
      ImageEnMView.MIO.Params[p-1].JPEG_Smooth := 100;
    end;
  end;
end;

procedure ConvertImage2Pdf(ImageFile, FilePdf: String; const JpegQuality: Integer = 90; const ResampleDpis: Integer = 0);   // Diference between 80 and 90 is not big ...
var
  ImageEnMView: TImageEnMView;
  p: Integer;
begin
  ImageEnMView := TImageEnMView.Create(Nil);

  ImageEnMView.LoadFromFileOnDemand(ImageFile, false);
  ConvertImage2Pdf(ImageEnMView, JpegQuality, ResampleDpis);

  ImageEnMView.MIO.SaveToFilePDF(FilePdf);
  ImageEnMView.Free;
end;

procedure ConvertImage2Tiff(ImageEnMView: TImageEnMView; const TIFF_Compression: TIOTIFFCompression = ioTIFF_G4FAX; const ResampleDpis: Integer = 0);
var
  p: Integer;
begin
  for p := 1 to ImageEnMView.ImageCount do
  begin
    ImageEnMView.SelectedImage := p-1;

    if ImageEnMView.IEBitmap.Width <> 0 then    // 2016-07-29 Force to load IEBitmap reading any property !!!
    begin
      if ResampleDpis <> 0 then
        if ImageEnMView.MIO.Params[p-1].Dpi <> ResampleDpis then
          ImageEnMView.Proc.Resample( Round(ImageEnMView.MIO.Params[p-1].Width * ResampleDpis / ImageEnMView.MIO.Params[p-1].Dpi),    // New width
                                     -1,   // New height (Auto) ...
                                     rfFastLinear, True);

      if TIFF_Compression = ioTIFF_G4FAX then
      begin
        ImageEnMView.MIO.Params[p-1].TIFF_Compression := ioTIFF_G4FAX;
        ImageEnMView.MIO.Params[p-1].BitsPerSample := 1;
        ImageEnMView.MIO.Params[p-1].SamplesPerPixel := 1;
      end
      else begin
        ImageEnMView.MIO.Params[p-1].BitsPerSample := 8;
        ImageEnMView.MIO.Params[p-1].SamplesPerPixel := 3;
        ImageEnMView.MIO.Params[p-1].TIFF_Compression := TIFF_Compression; // Old ... ioTIFF_LZW;
      end;
    end;
  end;
end;

procedure ConvertImage2Tiff(ImageFile, FileTiff: String; const TIFF_Compression: TIOTIFFCompression = ioTIFF_G4FAX; const ResampleDpis: Integer = 0);
var
  ImageEnMView: TImageEnMView;
  p: Integer;
begin
  ImageEnMView := TImageEnMView.Create(Nil);

  ImageEnMView.LoadFromFileOnDemand(ImageFile, false);
  ConvertImage2Tiff(ImageEnMView, TIFF_Compression, ResampleDpis);

  ImageEnMView.MIO.SaveToFileTIFF(FileTiff);
  ImageEnMView.Free;
end;

// Because imageen components don' t save always in first try if file exists :
// On va limiter la tentative de sauvegarde  100 fois
function SaveToTiff(ImageEnMView: TImageEnMView; const DestFileTiff: String): Boolean;
var
  TryCount: Integer;
begin
  Result := false;
  TryCount := 0;

  repeat
    try
      ImageEnMView.MIO.SaveToFileTIFF(DestFileTiff);
      Result := true;
    except
      Inc(TryCount);
      Sleep(100);
    end;
  until Result or (TryCount >= 100);
end;

function SaveToPdf(ImageEnMView: TImageEnMView; const DestFilePdf: String): Boolean;
var
  TryCount: Integer;
begin
  Result := false;
  TryCount := 0;

  repeat
    try
      ImageEnMView.MIO.SaveToFilePDF(DestFilePdf);
      Result := true;
    except
      Inc(TryCount);
      Sleep(100);
    end;
  until Result or (TryCount >= 100);
end;

(* OLD function ConvertTiffToTIFFG4FAX(SrcFileTIF, DestFileTIF: String; var Modified: Boolean): Boolean;
var
  ImageEnMView, ImageEnMViewDest: TImageEnMView;
  i: Integer;
begin
  Result := false;
  Modified := false;
  ImageEnMView := TImageEnMView.Create(Nil);
  ImageEnMViewDest := TImageEnMView.Create(Nil);

  try
    ImageEnMView.MIO.LoadFromFile(SrcFileTIF);

    if ImageEnMView.MIO.ParamsCount > 0 then
      if ImageEnMView.MIO.Params[0].TIFF_Compression <> ioTIFF_G4FAX then
        for i := 0 to ImageEnMView.MIO.ParamsCount-1 do
        begin
          Modified := True;
          ImageEnMView.SelectedImage := i;
          ImageEnMView.Proc.ConvertToBWThreshold;
          // BUG On peut pas modifier TIFF_Compression := ioTIFF_G4FAX et BitsPerSample := 1 en mme temps !!!
          //ImageEnMView.MIO.Params[i].TIFF_Compression := ioTIFF_G4FAX;
          ImageEnMView.MIO.Params[i].BitsPerSample := 1;
          ImageEnMView.MIO.Params[i].SamplesPerPixel := 1;

          ImageEnMViewDest.AppendImage;
          ImageEnMViewDest.SetImage(i, ImageEnMView.Bitmap);
          ImageEnMViewDest.MIO.Params[i].TIFF_Compression := ioTIFF_G4FAX;
          ImageEnMViewDest.MIO.Params[i].BitsPerSample := 1;
          ImageEnMViewDest.MIO.Params[i].SamplesPerPixel := 1;
        end;

    if Modified then
      SaveToTiff(ImageEnMViewDest, DestFileTIF);

    Result := True;
  except
  end;

  ImageEnMView.Free;
  ImageEnMViewDest.Free;
end;    *)


function EncodeTiffToG4FAX(const SrcFileTIF, DestFileTIF: String; var Modified: Boolean; const BWLocalThresholdParam: Integer = 16): Boolean;
var
  ImageEnMView, ImageEnMViewDest: TImageEnMView;
  i: Integer;
begin
  Result := false;
  Modified := false;
  ImageEnMView := TImageEnMView.Create(Nil);
  ImageEnMViewDest := TImageEnMView.Create(Nil);

  try
    ImageEnMView.MIO.LoadFromFileTiff(SrcFileTIF);

    if ImageEnMView.MIO.ParamsCount > 0 then
      if ImageEnMView.MIO.Params[0].TIFF_Compression <> ioTIFF_G4FAX then
        for i := 0 to ImageEnMView.MIO.ParamsCount-1 do
        begin
          Modified := True;
          ImageEnMView.SelectedImage := i;

          if BWLocalThresholdParam = 0
          then ImageEnMView.Proc.ConvertToBWThreshold
          else ImageEnMView.Proc.ConvertToBWLocalThreshold(BWLocalThresholdParam, ietMean, BWLocalThresholdParam);   //       (ietMean, ietMedian, ietMeanMinMax);


          // BUG On peut pas modifier TIFF_Compression := ioTIFF_G4FAX et BitsPerSample := 1 en mme temps !!!
          //ImageEnMView.MIO.Params[i].TIFF_Compression := ioTIFF_G4FAX;

// 2016-07-18 ... Not needed :
//          ImageEnMView.MIO.Params[i].BitsPerSample := 1;
//          ImageEnMView.MIO.Params[i].SamplesPerPixel := 1;

          ImageEnMViewDest.AppendImage;
          ImageEnMViewDest.SetImage(i, ImageEnMView.Bitmap);
          ImageEnMViewDest.MIO.Params[i].TIFF_Compression := ioTIFF_G4FAX;
          ImageEnMViewDest.MIO.Params[i].BitsPerSample := 1;
          ImageEnMViewDest.MIO.Params[i].SamplesPerPixel := 1;
        end;

    Result := True;
  except
    // Le fichier n' a pas pu tre ouvert/sauvegard ...
  end;

  ImageEnMView.Free;

  if Result and Modified then
    cyImageEn.SaveToTiff(ImageEnMViewDest, DestFileTIF);  // Impossible de sauvegarder par dessus l' ancien !

  ImageEnMViewDest.Free;
end;

end.
