{   Unit cyDebenuPDFLibrary

    Description:
    Unit with functions to use with VirtualTree components: http://www.soft-gems.net/index.php/controls/virtual-treeview.

    $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    $   Accept any PAYPAL DONATION $$$  
    $      to: mauricio_box@yahoo.com      
    

    * ***** BEGIN LICENSE BLOCK *****
    *
    * Version: MPL 1.1
    *
    * The contents of this file are subject to the Mozilla Public License Version
    * 1.1 (the "License"); you may not use this file except in compliance with the
    * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
    *
    * Software distributed under the License is distributed on an "AS IS" basis,
    * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
    * the specific language governing rights and limitations under the License.
    *
    * The Initial Developer of the Original Code is Mauricio
    * (https://sourceforge.net/projects/tcycomponents/).
    *
    * Donations: see Donation section on Description.txt
    *
    * Alternatively, the contents of this file may be used under the terms of
    * either the GNU General Public License Version 2 or later (the "GPL"), or the
    * GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
    * case the provisions of the GPL or the LGPL are applicable instead of those
    * above. If you wish to allow use of your version of this file only under the
    * terms of either the GPL or the LGPL, and not to allow others to use your
    * version of this file under the terms of the MPL, indicate your decision by
    * deleting the provisions above and replace them with the notice and other
    * provisions required by the LGPL or the GPL. If you do not delete the
    * provisions above, a recipient may use your version of this file under the
    * terms of any one of the MPL, the GPL or the LGPL.
    *
    * ***** END LICENSE BLOCK *****}

unit cyVirtualTrees;

interface

uses Classes, SysUtils, cyStrUtils, VirtualTrees;

type
  TGetNodeTextEvent = procedure (Node: Pointer; var ResultText: String);

procedure VirtualStringTree_Filter(VirtualST: TVirtualStringTree; const Input: string; GetNodeTextEvent: TGetNodeTextEvent; const StrFilterOptions: TStrFilterOptions = [strfoCaseInsensitive,strfoAccentsInsensitive]);

implementation

procedure VirtualStringTree_Filter(VirtualST: TVirtualStringTree; const Input: string; GetNodeTextEvent: TGetNodeTextEvent; const StrFilterOptions: TStrFilterOptions = [strfoCaseInsensitive,strfoAccentsInsensitive]);
var
  ParentNode, CurrentNode: PVirtualNode;
  NodeText: String;
  NodeVisible: Boolean;

      procedure HandleChildsNode(ParentNode: PVirtualNode; var RsltAnyChildVisible: Boolean);
      var
        ParentNodeLevel, CurrentNodeLevel: Integer;
        ChildNode: PVirtualNode;
        ChildVisible: Boolean;
      begin
        RsltAnyChildVisible := false;
        ParentNodeLevel := VirtualST.GetNodeLevel(ParentNode);
        CurrentNode := VirtualST.GetNext(ParentNode);

        while Assigned(CurrentNode) do
        begin
          CurrentNodeLevel := VirtualST.GetNodeLevel(CurrentNode);

          if CurrentNodeLevel > ParentNodeLevel then
          begin
            ChildNode := CurrentNode;
            HandleChildsNode(CurrentNode, ChildVisible);

            if not ChildVisible then
              if Input <> '' then
              begin
                GetNodeTextEvent(VirtualST.GetNodeData(ChildNode), NodeText);
                ChildVisible := String_MatchInput(NodeText, Input, StrFilterOptions);
              end
              else
                ChildVisible := true;

            VirtualST.IsVisible[ChildNode] := ChildVisible;

            RsltAnyChildVisible := RsltAnyChildVisible or ChildVisible;
          end
          else
            Break;
        end;
      end;


begin
  VirtualST.BeginUpdate;

  ParentNode := VirtualST.GetFirst;

  while Assigned(ParentNode) do
  begin
    HandleChildsNode(ParentNode, NodeVisible);

    if not NodeVisible then
      if Input <> '' then
      begin
        GetNodeTextEvent(VirtualST.GetNodeData(ParentNode), NodeText);
        NodeVisible := String_MatchInput(NodeText, Input, StrFilterOptions);
      end
      else
        NodeVisible := true;

    VirtualST.IsVisible[ParentNode] := NodeVisible;

    ParentNode := CurrentNode;
  end;

  VirtualST.EndUpdate;
end;

end.
