{ cyDER  Components
  Description: Document Elements Recognition classes

  Todo:
  - Determine new Expression properties like isCellGrid: Boolean.
}

unit cyDocER;

interface

uses
  Classes, Windows, Controls, SysUtils, cyStrUtils, cyDERUtils, VCL.cyGraphics, cyDateUtils;

type
  TReadingDirection = (rdLeft, rdTop, rdRight, rdBottom);
  TReadingDirections = set of TReadingDirection;

  TOCRCarInfo = record
    Value: String;
    PageNumber: Integer;
    Left: Integer;
    Top: Integer;
    Right: Integer;
    Bottom: Integer;
    WordNumber: Integer;
  end;

  TcyDocER = class;

  TOCRPage = class
  private
    FTag: Integer;
    FPixelsWidth: Integer;
    FPixelsHeight: Integer;
    FResolution: Integer;
    FAngleRotation: Integer;
    FTagStr: String;
    procedure SetTagStr(const Value: String);
  protected
  public
    property Tag: Integer read FTag write FTag;
    property TagStr: String read FTagStr write SetTagStr;
    property AngleRotation: Integer read FAngleRotation write FAngleRotation;
    property PixelsWidth: Integer read FPixelsWidth write FPixelsWidth;
    property PixelsHeight: Integer read FPixelsHeight write FPixelsHeight;
    property Resolution: Integer read FResolution write FResolution;
  end;

  TOCRExpression = class
  private
    FValue: String;
    FAssociatedElementIndex: Integer;
    FAssociatedElementKeywordIndex: Integer;
    FAssociatedExpressionKeywordIndex: Integer; // Connexion to the expression's keyword found!
    FRecognizedFloat: String;
    FRecognizedNumbers: String;
    FRecognizedInteger: String;
    FRecognizedPercentage: String;
    FRecognizedWebMail: String;
    FRecognizedWebsite: String;
    FRecognizedMoney: String;
    FRecognizedDate: String;
    FRightPxPos: Integer;
    FBottomPxPos: Integer;
    FTopPxPos: Integer;
    FLeftPxPos: Integer;
    FPageNumber: Integer;
    FDERValue: DERString;
    FRecognizedType: TElementsType;
    FOCRConfidence: Extended;
  protected
  public
    // OCR information :
    function RecognizedValue: String;
    function RecognizedFloatvalue: Extended;
    function RecognizedDateValue: TDate;
    property LeftPxPos: Integer read FLeftPxPos;
    property TopPxPos: Integer read FTopPxPos;
    property RightPxPos: Integer read FRightPxPos;
    property BottomPxPos: Integer read FBottomPxPos;
    property Value: String read FValue write FValue;
    property PageNumber: Integer read FPageNumber;
    // DER information :
    property DERValue: DERString read FDERValue;
    property AssociatedElementIndex: Integer read FAssociatedElementIndex write FAssociatedElementIndex;
    property AssociatedElementKeywordIndex: Integer read FAssociatedElementKeywordIndex write FAssociatedElementKeywordIndex;
    property AssociatedExpressionKeywordIndex: Integer read FAssociatedExpressionKeywordIndex write FAssociatedExpressionKeywordIndex;
    property OCRConfidence: Extended read FOCRConfidence write FOCRConfidence;
    property RecognizedType: TElementsType read FRecognizedType write FRecognizedType;
    property RecognizedDate: String read FRecognizedDate write FRecognizedDate;
    property RecognizedMoney: String read FRecognizedMoney write FRecognizedMoney;
    property RecognizedWebsite: String read FRecognizedWebsite write FRecognizedWebsite;
    property RecognizedWebMail: String read FRecognizedWebMail write FRecognizedWebMail;
    property RecognizedPercentage: String read FRecognizedPercentage write FRecognizedPercentage;
    property RecognizedFloat: String read FRecognizedFloat write FRecognizedFloat;
    property RecognizedInteger: String read FRecognizedInteger write FRecognizedInteger;
    property RecognizedNumbers: String read FRecognizedNumbers write FRecognizedNumbers;
  end;



  TSearchValueLocation = (slFromLeft, slFromTop, slFromRight, slFromBottom);
  TPatternPositionMode = (ppFromTopLeftPage, ppFromTopLeftKeyword);
  TPatternPageNumberMode = (ppFromBeginning, ppFromEnd);

  TElement = class(TCollectionItem)
  private
    FTag: Integer;
    FKeyWords: TStrings;
    FPatternFromLeftMm: Extended;
    FPatternFromTopMm: Extended;
    FPatternToRightMm: Extended;
    FPatternToBottomMm: Extended;
    FPatternValueLocation: TSearchValueLocation;
    FPatternValueNumber: Integer;
    FValueCount: Integer;
    FPatternPageNumber: Integer;
    FPatternPageNumberMode: TPatternPageNumberMode;
    FValueType: TElementsType;
    FPatternPositionMode: TPatternPositionMode;
    fMask: string;
    procedure SetKeyWords(const Value: TStrings);
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function PatternRecognition: boolean;
  published
    property KeyWords: TStrings read FKeyWords write SetKeyWords;
    property Mask: string read fMask write fMask;
    property PatternPageNumber: Integer read FPatternPageNumber write FPatternPageNumber default 0;
    property PatternPageNumberMode: TPatternPageNumberMode read FPatternPageNumberMode write FPatternPageNumberMode default ppFromBeginning;
    property PatternFromLeftMm: Extended read FPatternFromLeftMm write FPatternFromLeftMm;
    property PatternFromTopMm: Extended read FPatternFromTopMm write FPatternFromTopMm;
    property PatternToRightMm: Extended read FPatternToRightMm write FPatternToRightMm;
    property PatternToBottomMm: Extended read FPatternToBottomMm write FPatternToBottomMm;
    property PatternPositionMode: TPatternPositionMode read FPatternPositionMode write FPatternPositionMode default ppFromTopLeftPage;
    property PatternValueLocation: TSearchValueLocation read FPatternValueLocation write FPatternValueLocation default slFromBottom;
    property PatternValueNumber: Integer read FPatternValueNumber write FPatternValueNumber default 1;   // Which value must be returned if more than one value in defined area
    property ValueCount: Integer read FValueCount write FValueCount default 1;          // Number of values that must be retrieved
    property ValueType: TElementsType read FValueType write FValueType default etMoney;
    property Tag: Integer read FTag write FTag default 0;
  end;

  TElementClass = class of TElement;

  TElements = Class(TCollection)
  private
    FOwner: TPersistent;
    FOnChange: TNotifyEvent;
    function GetElement(Index: Integer): TElement;
  protected
    function GetOwner: TPersistent; Override;
    procedure Update(Item: TCollectionItem); Override;
  public
    constructor Create(aDocument: TcyDocER; ElementClass: TElementClass);
    function Add: TElement;
    procedure Delete(Index: Integer);
    property Items[Index: Integer]: TElement read GetElement; default;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TRecognitionElementsMode = (reKeywords, reValues, reKeywordsAndValues);
  TRecognitionPriorityMode = (rpKeywordsLenth, rpSinglelineKeywordsLength);

  TRecognitionOption = (roKeywordsByPriority, roSmartNumbersRec, roSmartWebsiteRec, roSmartKeywordRec);
  TRecognitionOptions = Set of TRecognitionOption;

  TExpressionOption = (eoDERValueAsString);
  TExpressionOptions = Set of TExpressionOption;

  TLocateExpressionOption = (lePartialKey, leRelativePositionKey, leInsensitive, leSmartKeywordRec);
  TLocateExpressionOptions = Set of TLocateExpressionOption;

  TValidateElementValueResult =(veValueOk, veInvalidValue, veInvalidValueStopSearching, veValueTooFar);

  TProcOnExpressionMerged = procedure (Sender: TObject; aExpressionIndex, toExpressionIndex: Integer) of object;
  TProcOnRecognizeElementKeyword = procedure (Sender: TObject; ElementIndex, ElementKeywordIndex: Integer) of object;
  TProcValidateElementValue = procedure (Sender: TObject; ElementIndex, ElementKeywordIndex, ExpressionKeywordIndex, ExpressionValueIndex: Integer; var ValidateElementValueResult: TValidateElementValueResult) of object;
  TProcValidateElementKeyword = procedure (Sender: TObject; ElementIndex, ElementKeywordIndex: Integer; ExpressionsList: TStrings; var Accept: Boolean) of object;
  TProcRetrieveElementValuesFromRect = procedure (Sender: TObject; ElementIndex, ElementKeywordIndex, ExpressionKeywordIndex: Integer; ExpressionList: TStrings) of object;

  // For individual component ... [ComponentPlatformsAttribute (pidWin32 or pidWin64)]
  TcyDocER = class(TComponent)
  private
    FPages: TList;      // TOCRPage objects list
    FExpressions: TList;  // TOCRExpression objects list
    FElements: TElements;
    FRecognitionOptions: TRecognitionOptions;
    FBeforeRecognizeElementsValues: TNotifyEvent;
    FBeforeRecognizeElementsKeywords: TNotifyEvent;
    FBeforeRecognizeElementKeyword: TProcOnRecognizeElementKeyword;
    FAfterRecognizeElementKeyword: TProcOnRecognizeElementKeyword;
    FOnValidateElementValue: TProcValidateElementValue;
    FOnValidateElementKeyword: TProcValidateElementKeyword;
    FRecognitionPriorityMode: TRecognitionPriorityMode;
    FOnRetrieveElementValuesFromRect: TProcRetrieveElementValuesFromRect;
    FOnExpressionMerged: TProcOnExpressionMerged;
    FShortMonthNames: TStrings;
    fAutoMergeExpressionsRatio: Extended;
    FExpressionOptions: TExpressionOptions;
    function GetPages(Index: Integer): TOCRPage;
    function GetPageCount: Integer;
    function GetExpressions(Index: Integer): TOCRExpression;
    function GetExpressionCount: Integer;
    procedure SetElements(const Value: TElements);
    procedure SetRecognitionOptions(const Value: TRecognitionOptions);
    procedure SetShortMonthNames(const Value: TStrings);
    procedure SetExpressionOptions(const Value: TExpressionOptions);
  protected
    procedure ExtractTesseractBoxlineInfo(const aLine: String; var RsltCar: String; var RsltPage, RsltLeft, RsltTop, RsltRight, RsltBottom: Integer);
    function RecognizeMonth(aDERString: DERString): Integer;
    function DoStringToDERCharSet(const aString: String): String;
  public
    Recognition_MinDate: Integer;
    Recognition_MaxDate: Integer;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    // OCR feeding :
    procedure LoadFromTesseractString(aString: String; const aPageNumber: Integer);
    procedure LoadFromTesseractFile(const aFilename: String; const aPageNumber: Word{$IFDEF UNICODE}; const aEncoding: TEncoding{$ENDIF});
    procedure LoadFromTesseractBoxStringList(aStrings: TStrings; const ImageDocumentPages, ImageResolution, ImagePixelsWidth, ImagePixelsHeight: Integer);
    procedure LoadFromTesseractBoxFile(const aFilename: String; {$IFDEF UNICODE} const aEncoding: TEncoding; {$ENDIF} const ImageDocumentPages, ImageResolution, ImagePixelsWidth, ImagePixelsHeight: Integer);
    procedure LoadFromStringList(aStrings: TStrings);
    // Load / Save :
    procedure LoadFromFile(const aFilename: String{$IFDEF UNICODE}; const aEncoding: TEncoding{$ENDIF});
    procedure SaveToStringList(aStrings: TStrings);
    procedure SaveToFile(const aFilename: String{$IFDEF UNICODE}; const aEncoding: TEncoding{$ENDIF});
    // Utilities :
    function GetPagePixelsWidth(const PageNumber: Integer; const FromBeginning: Boolean = true): Integer;
    function GetPagePixelsHeight(const PageNumber: Integer; const FromBeginning: Boolean = true): Integer;
    function GetPageResolution(const PageNumber: Integer; const FromBeginning: Boolean = true): Integer;
    function MmToPx(const MmValue: Extended; const Resolution: Integer): Integer;
    function PxToMm(const pxValue: Extended; const Resolution: Integer): Extended;
    function MmToPxFromPage(const MmValue: Extended; const PageNumber: Integer): Integer;
    function PxToMmFromPage(const pxValue: Extended; const PageNumber: Integer): Extended;
    procedure ClearPages;
    procedure ClearExpressions;
    function GetOCRText(const FromPage: Integer = 0): String;
    function GetOCRTextFromRect(const FromRect: TRect; const FromPage: Integer; const AllowPartiallyInside: Boolean = true): String;
    function GetAsDocumentOCRText(const FromPage: Integer = 0): String;
    function GetAsDocumentOCRTextFromRect(const FromRect: TRect; const FromPage: Integer; const AllowPartiallyInside: Boolean = true): String;
    function NewPage(PageResolution, PagePixelsWidth, PagePixelsHeight: Integer; const PageAngleRotation: integer = 0; const PageTag: Integer = 0; const PageTagStr: String = ''): Integer;
    procedure RotatePageExpressions(const PageNumber, PageWidthBeforeRotation, PageHeightBeforeRotation: Integer; const ToRight: Boolean);
    function ExpressionInRect(const aExpressionIndex: Integer; const InRect: TRect; const AllowPartiallyInside: Boolean = true): Boolean;
    procedure RecognizeExpressionType(const aExpressionIndex: Integer);
    function AddExpression(const aString: String; const aPageNumber: Word; const aOCRConfidence: Extended; const aRect: TRect): Integer; overload;
    function CloneExpression(const aExpressionIndex: Integer): Integer;
    function AddExpression(const aString: String; const aPageNumber: Word; const aOCRConfidence: Extended): Integer; overload;
    procedure ExpressionAdded;
    procedure ExpressionLoaded;
    procedure DeleteExpression(const aExpressionIndex: Integer);
    function LocateExpression(Value: String; const FromIndex, ToIndex, MaxCarErrors: Integer; const Options: TLocateExpressionOptions; var RsltCarPos: Integer): Integer;
    function ExpressionsInSameLine(const aExpressionIndex1, aExpressionIndex2: Integer): Boolean;
    procedure GetAroundExpressions(const aExpressionIndex, MaxCarErrors, ScopePx: Integer; const SearchValue: String; const SearchOptions: TLocateExpressionOptions; var RsltLeft, RsltTop, RsltRight, RsltBottom: Integer);
    function MergeExpressions(aExpressionList: TStrings): Integer; overload;
    procedure MergeExpressions(aExpressionIndex, ToExpressionIndex: Integer; const SeparWithSpace: boolean = true); overload;
    function ExpressionsSideBySide(const ExpressionIndexAtLeft, ExpressionIndexAtRight, MaxPxSpacing: Integer): Boolean;
    function IsElementKeyword(const aElementIndex: Integer; aStr: String): Integer;
    function IsExpressionsSameValue(const ExpressionIndex1, ExpressionIndex2: Integer): Boolean;
    function FindExpression(const StartIndex, AssociatedElementIndex, AssociatedElementKeywordIndex, AssociatedExpressionKeywordIndex: Integer; const IncludeElementsTypes, ExcludeElementsTypes: TElementsTypes): Integer; overload;
    function FindExpression(const Value: Variant; const ValueType: TElementsType; const FromIndex, ToIndex: Integer): Integer; overload;
    procedure DissociateExpressions(const FromElementIndex, FromElementKeywordIndex: Integer);
    procedure InitializeRecognition;
    function RecognizeElementKeyword(const ElementIndex, KeywordIndex: Integer; const InlineKeyword: Boolean): Boolean;
    function RecognizeElementValuesFromRect(const OfElementIndex, OfElementKeywordIndex, OfExpressionKeywordIndex: Integer; const aValueType: TElementsType;
     const aPageNumber, LeftPos, TopPos, RightPos, BottomPos: Integer; const ValueLocation: TSearchValueLocation;
     ValueNumber, ValueCount: Integer; const PatternMode: Boolean; var UserAbort: Boolean): Integer;
    procedure RecognizeLongDates;
    procedure RecognizeElementValues(ElementIndex: Integer);
    procedure RecognizeElementsKeywords;
    procedure RecognizeElementsValues;
    procedure RecognizeElements;
    // Properties :
    property ExpressionCount: Integer read GetExpressionCount;
    property Expressions[Index: Integer]: TOCRExpression read GetExpressions; default;
    property PageCount: Integer read GetPageCount;
    property Pages[Index: Integer]: TOCRPage read GetPages;
  published
    property AutoMergeExpressionsRatio: Extended read fAutoMergeExpressionsRatio write fAutoMergeExpressionsRatio;
    property ShortMonthNames: TStrings read FShortMonthNames write SetShortMonthNames;
    property Elements: TElements read FElements write SetElements;
    property ExpressionOptions: TExpressionOptions read FExpressionOptions write SetExpressionOptions default [];
    property RecognitionOptions: TRecognitionOptions read FRecognitionOptions write SetRecognitionOptions default [roSmartNumbersRec, roSmartWebsiteRec, roSmartKeywordRec, roKeywordsByPriority];
    property RecognitionPriorityMode: TRecognitionPriorityMode read FRecognitionPriorityMode write FRecognitionPriorityMode default rpSinglelineKeywordsLength;
    property BeforeRecognizeElementsKeywords: TNotifyEvent read FBeforeRecognizeElementsKeywords write FBeforeRecognizeElementsKeywords;
    property BeforeRecognizeElementsValues: TNotifyEvent read FBeforeRecognizeElementsValues write FBeforeRecognizeElementsValues;
    property BeforeRecognizeElementKeyword: TProcOnRecognizeElementKeyword read FBeforeRecognizeElementKeyword write FBeforeRecognizeElementKeyword;
    property AfterRecognizeElementKeyword: TProcOnRecognizeElementKeyword read FAfterRecognizeElementKeyword write FAfterRecognizeElementKeyword;
    property OnValidateElementKeyword: TProcValidateElementKeyword read FOnValidateElementKeyword write FOnValidateElementKeyword;
    property OnRetrieveElementValuesFromRect: TProcRetrieveElementValuesFromRect read FOnRetrieveElementValuesFromRect write FOnRetrieveElementValuesFromRect;
    property OnValidateElementValue: TProcValidateElementValue read FOnValidateElementValue write FOnValidateElementValue;
    property OnExpressionMerged: TProcOnExpressionMerged read FOnExpressionMerged write FOnExpressionMerged;
  end;

implementation

{ TOCRPage }
procedure TOCRPage.SetTagStr(const Value: String);
begin
  FTagStr := Value;

  if FTagStr <> '' then
    String_SubstFast(';', '', FTagStr);
end;

{ TOCRExpression }
function TOCRExpression.RecognizedFloatValue: Extended;
begin
  Result := 0;

  if FRecognizedFloat <> '' then
    Result := StrToFloat(FRecognizedFloat, cyDERUtils.LocalFormatSettings);
end;

function TOCRExpression.RecognizedDateValue: TDate;
begin
  Result := 0;

  if FRecognizedDate <> '' then
    Result := StrToInt(FRecognizedDate);
end;

function TOCRExpression.RecognizedValue: String;
begin
  case FRecognizedType of
    etText:              Result := FValue;
    etTextLine:          Result := FValue;
    etParagraph:         Result := FValue;
    etExpressionKeyWord: Result := FValue;
    etNumbers:           Result := FRecognizedNumbers;
    etInteger:           Result := FRecognizedInteger;
    etFloat:             Result := FRecognizedFloat;
    etPercentage:        Result := FRecognizedPercentage;
    etwebSite:           Result := FRecognizedWebsite;
    etWebMail:           Result := FRecognizedWebMail;
    etMoney:             Result := FRecognizedMoney;
    etDate:              Result := FRecognizedDate;
  end;
end;

{ TElement }
constructor TElement.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FKeyWords := TStringList.Create;
  FPatternPageNumber := 0;
  PatternPageNumberMode := ppFromBeginning;
  FPatternFromLeftMm := 0;
  FPatternFromTopMm := 0;
  FPatternToRightMm := 0;
  FPatternToBottomMm := 0;
  FPatternPositionMode := ppFromTopLeftPage;
  FPatternValueLocation := slFromBottom;
  FPatternValueNumber := 1;
  FValueCount := 1;
  FValueType := etMoney;
  FTag := 0;
end;

destructor TElement.Destroy;
begin
  FKeyWords.Free;
  inherited;
end;

function TElement.GetDisplayName: string;
var k: Integer;
begin
  Result := '';

  for k := 0 to FKeyWords.Count-1 do
    if Result = ''
    then Result := FKeyWords[k]
    else Result := Result + ', ' + FKeyWords[k];
end;

function TElement.PatternRecognition: boolean;
begin
  Result := (FPatternFromLeftMm <> 0) or (FPatternFromTopMm <> 0) or (FPatternToRightMm <> 0) or (FPatternToBottomMm <> 0);
end;

procedure TElement.SetKeyWords(const Value: TStrings);
begin
  if Assigned(FKeyWords) then
    FKeyWords.Assign(Value)
  else
    FKeyWords := Value;
end;

procedure TElement.Assign(Source: TPersistent);
begin
  if Source is TElement then
  begin
    FKeyWords  := TElement(Source).FKeyWords;
    FPatternPageNumber := TElement(Source).FPatternPageNumber;
    FPatternPageNumberMode := TElement(Source).FPatternPageNumberMode;
    FPatternFromLeftMm := TElement(Source).FPatternFromLeftMm;
    FPatternFromTopMm := TElement(Source).FPatternFromTopMm;
    FPatternToRightMm := TElement(Source).FPatternToRightMm;
    FPatternToBottomMm := TElement(Source).FPatternToBottomMm;
    FPatternPositionMode := TElement(Source).FPatternPositionMode;
    FPatternValueLocation := TElement(Source).FPatternValueLocation;
    FPatternValueNumber := TElement(Source).FPatternValueNumber;
    FValueCount := TElement(Source).FValueCount;
    FValueType := TElement(Source).FValueType;
    FTag := TElement(Source).FTag;
  end;
//  inherited Assign(Source);
end;

{ TElements }
constructor TElements.Create(aDocument: TcyDocER; ElementClass: TElementClass);
begin
  inherited Create(ElementClass);
  FOwner := aDocument;
end;

function TElements.Add: TElement;
begin
  Result := TElement(inherited Add);
  Result.Changed(false);      // It will call TcyElements.Update only at run-time!
end;

procedure TElements.Delete(Index: Integer);
begin
  Inherited;
  Update(Nil);
end;

function TElements.GetElement(Index: Integer): TElement;
begin
  Result := TElement(inherited Items[Index]);
end;

function TElements.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

// Event Called by setting properties/events of TElement :
procedure TElements.Update(Item: TCollectionItem);
begin
  inherited;
  if Assigned(FOnChange) then
    FOnChange(Self);
end;



{ TcyDocER }
constructor TcyDocER.Create(AOwner: TComponent);
var
  m: Integer;
  {$IFDEF DCC} // Delphi XE2/XE3 Win 32/64
  fs: TFormatSettings;
  {$ENDIF DCC}
begin
  inherited Create(AOwner);

  Recognition_MinDate := 1970;
  Recognition_MaxDate := 2020;

  FPages := TList.Create;
  FExpressions := TList.Create;
  FShortMonthNames := TStringList.Create;

  {$IFDEF DCC} // Delphi XE2/XE3 Win 32/64
  fs := TFormatSettings.Create;
  for m := 1 to 12 do
    FShortMonthNames.Add(fs.ShortMonthNames[m]);
  {$ELSE} // Delphi other
  for m := 1 to 12 do
    FShortMonthNames.Add(SysUtils.ShortMonthNames[m]);
  {$ENDIF DCC}

  FElements := TElements.Create(self, TElement);
  FAutoMergeExpressionsRatio := 1.5;
  FExpressionOptions := [];
  FRecognitionOptions := [roKeywordsByPriority, roSmartNumbersRec, roSmartWebsiteRec, roSmartKeywordRec];
  FRecognitionPriorityMode := rpSinglelineKeywordsLength;
end;

destructor TcyDocER.Destroy;
begin
  ClearPages;
  ClearExpressions;
  FShortMonthNames.Free;
  FElements.Free;
  FElements := Nil;
  FPages.Free;
  FExpressions.Free;
  inherited;
end;

procedure TcyDocER.DissociateExpressions(const FromElementIndex, FromElementKeywordIndex: Integer);
var i: Integer;
begin
  for i := 0 to ExpressionCount-1 do
    if (Expressions[i].FAssociatedElementIndex = FromElementIndex) or (FromElementIndex = -1) then
      if (Expressions[i].FAssociatedElementKeywordIndex = FromElementKeywordIndex) or (FromElementKeywordIndex = -1) then
      begin
        Expressions[i].FAssociatedElementIndex := -1;
        Expressions[i].FAssociatedElementKeywordIndex := -1;
        Expressions[i].FAssociatedExpressionKeywordIndex := -1;
        if Expressions[i].FRecognizedType = etExpressionKeyWord then
          RecognizeExpressionType(i);
      end;
end;

function TcyDocER.DoStringToDERCharSet(const aString: String): String;
begin
  if eoDERValueAsString in FExpressionOptions then
  begin
    Result := aString;
    String_SubstFast(',', '.', Result);  // For float recognition ...
    String_SubstFast(' ', '', Result);  // For float recognition, after merging 2 expressions ...
  end
  else
    Result := StringToDERCharSet(aString);
end;

procedure TcyDocER.SetShortMonthNames(const Value: TStrings);
begin
  if Assigned(FShortMonthNames) then
    FShortMonthNames.Assign(Value)
  else
    FShortMonthNames := Value;
end;

procedure TcyDocER.SetElements(const Value: TElements);
begin
  FElements := Value;
end;

procedure TcyDocER.SetExpressionOptions(const Value: TExpressionOptions);
begin
  FExpressionOptions := Value;
end;

procedure TcyDocER.SetRecognitionOptions(const Value: TRecognitionOptions);
begin
  FRecognitionOptions := Value;
end;

procedure TcyDocER.ClearPages;
var i: Integer;
begin
  // Free Pages :
  for i := 0 to PageCount-1 do
    try
      Pages[i].Free;
    except
    end;

  FPages.Clear;
end;

procedure TcyDocER.ClearExpressions;
var i: Integer;
begin
  // Free Expressions :
  for i := 0 to ExpressionCount-1 do
    try
      Expressions[i].Free;
    except
    end;

  FExpressions.Clear;
end;

function TcyDocER.GetPageCount: Integer;
begin
  Result := FPages.Count;
end;

function TcyDocER.GetPages(Index: Integer): TOCRPage;
begin
  Result := FPages[Index];
end;

function TcyDocER.GetExpressionCount: Integer;
begin
  Result := FExpressions.Count;
end;

function TcyDocER.GetExpressions(Index: Integer): TOCRExpression;
begin
  Result := FExpressions[Index];
end;

procedure TcyDocER.RecognizeExpressionType(const aExpressionIndex: Integer);
var
  _Numbers, _Integer, _Float, _Percentage, _webSite, _WebMail, _Money, _Date: String;
begin
  Expressions[aExpressionIndex].FDERValue := DoStringToDERCharSet(Expressions[aExpressionIndex].FValue);

  Expressions[aExpressionIndex].FRecognizedType := DERExecute(Expressions[aExpressionIndex].FDERValue,_Numbers,_Integer,_Float,_Percentage,_webSite,_WebMail,_Money,_Date,
                                                      (roSmartNumbersRec in FRecognitionOptions), roSmartWebsiteRec in FRecognitionOptions);
  Expressions[aExpressionIndex].FRecognizedDate       := _Date;
  Expressions[aExpressionIndex].FRecognizedMoney      := _Money;
  Expressions[aExpressionIndex].FRecognizedWebsite    := _Website;
  Expressions[aExpressionIndex].FRecognizedWebMail    := _WebMail;
  Expressions[aExpressionIndex].FRecognizedPercentage := _Percentage;
  Expressions[aExpressionIndex].FRecognizedFloat      := _Float;
  Expressions[aExpressionIndex].FRecognizedInteger    := _Integer;
  Expressions[aExpressionIndex].FRecognizedNumbers    := _Numbers;
end;

function TcyDocER.AddExpression(const aString: String; const aPageNumber: Word; const aOCRConfidence: Extended; const aRect: TRect): Integer;
begin
  Result := AddExpression(aString, aPageNumber, aOCRConfidence);

  if Result <> -1 then
  begin
    Expressions[Result].FLeftPxPos := aRect.Left;
    Expressions[Result].FTopPxPos := aRect.Top;
    Expressions[Result].FRightPxPos := aRect.Right;
    Expressions[Result].FBottomPxPos := aRect.Bottom;

    ExpressionAdded;  // Merge Expression in some cases ...
  end;
end;

function TcyDocER.CloneExpression(const aExpressionIndex: Integer): Integer;
begin
  Result := AddExpression(Expressions[aExpressionIndex].Value, Expressions[aExpressionIndex].PageNumber, Expressions[aExpressionIndex].OCRConfidence);

  if Result <> -1 then
  begin
    Expressions[Result].FLeftPxPos := Expressions[aExpressionIndex].FLeftPxPos;
    Expressions[Result].FTopPxPos := Expressions[aExpressionIndex].FTopPxPos;
    Expressions[Result].FRightPxPos := Expressions[aExpressionIndex].FRightPxPos;
    Expressions[Result].FBottomPxPos := Expressions[aExpressionIndex].FBottomPxPos;

    // No merge ... ExpressionAdded;
  end;
end;

function TcyDocER.AddExpression(const aString: String; const aPageNumber: Word; const aOCRConfidence: Extended): Integer;
var
  NewExpression: TOCRExpression;
begin
  Result := -1;
  if aString = '' then Exit;
  if DoStringToDERCharSet(aString) = '' then Exit;

  NewExpression := TOCRExpression.Create;
  FExpressions.Add(NewExpression);
  Result := FExpressions.Count - 1;

  Expressions[Result].FValue := aString;
  Expressions[Result].FPageNumber := aPageNumber;
  Expressions[Result].FAssociatedElementIndex := -1;
  Expressions[Result].FAssociatedElementKeywordIndex := -1;
  Expressions[Result].FAssociatedExpressionKeywordIndex := -1;
  Expressions[Result].FLeftPxPos := -1;
  Expressions[Result].FTopPxPos := -1;
  Expressions[Result].FRightPxPos := -1;
  Expressions[Result].FBottomPxPos := -1;
  Expressions[Result].FOCRConfidence := aOCRConfidence;

  // Recognize :
  RecognizeExpressionType(Result);

  // ExpressionAdded() No need to call here because we don' t have pxPos ...
end;

procedure TcyDocER.DeleteExpression(const aExpressionIndex: Integer);
begin
  if aExpressionIndex < 0 then
    raise Exception.Create('Cannot remove expression index below 0!');

  if aExpressionIndex > FExpressions.Count - 1 then
    raise Exception.Create('Cannot remove expression index above max.!');

  // Free stored object :
  Expressions[aExpressionIndex].Free;

  // Delete from list :
  FExpressions.Delete(aExpressionIndex);
end;

function TcyDocER.NewPage(PageResolution, PagePixelsWidth, PagePixelsHeight: Integer; const PageAngleRotation: integer = 0; const PageTag: Integer = 0; const PageTagStr: String = ''): Integer;
var
  aPage: TOCRPage;
begin
  Result := -1;

  aPage := TOCRPage.Create;
  FPages.Add(aPage);
  Result := FPages.Count - 1;

  // 2016-04-06 Can' t be 0 :
  if PagePixelsHeight = 0 then PagePixelsHeight := 3508;
  if PagePixelsWidth = 0 then PagePixelsWidth := 2480;
  if PageResolution = 0 then PageResolution := 300;

  Pages[Result].FTag           := PageTag;
  Pages[Result].FTagStr        := PageTagStr;
  Pages[Result].FPixelsWidth   := PagePixelsWidth;
  Pages[Result].FPixelsHeight  := PagePixelsHeight;
  Pages[Result].FResolution    := PageResolution;
  Pages[Result].FAngleRotation := PageAngleRotation;
end;

// Retrieve Tesseract line information into variables :
procedure TcyDocER.ExtractTesseractBoxlineInfo(const aLine: String; var RsltCar: String; var RsltPage, RsltLeft, RsltTop, RsltRight, RsltBottom: Integer);
var
  PagePixelsHeight: Integer;
begin
  try
    RsltCar    := (aLine + ' ')[1];
    RsltPage   := StrToInt( SubString_Get(aLine, ' ', 6) ) + 1;
    RsltLeft   := StrToInt( SubString_Get(aLine, ' ', 2) );

    if (RsltPage > 0) and (RsltPage <= PageCount)
    then PagePixelsHeight := Pages[RsltPage-1].FPixelsHeight
    else PagePixelsHeight := 3508;

    RsltBottom := PagePixelsHeight - StrToInt( SubString_Get(aLine, ' ', 3) );
    RsltRight  := StrToInt( SubString_Get(aLine, ' ', 4) );
    RsltTop    := PagePixelsHeight - StrToInt( SubString_Get(aLine, ' ', 5) );
  except
  end;
end;

function TcyDocER.FindExpression(const StartIndex, AssociatedElementIndex, AssociatedElementKeywordIndex, AssociatedExpressionKeywordIndex: Integer;
  const IncludeElementsTypes, ExcludeElementsTypes: TElementsTypes): Integer;
var i: Integer;
begin
  Result := -1;
  for i := StartIndex to ExpressionCount-1 do
    if (AssociatedElementIndex = -1) or (Expressions[i].AssociatedElementIndex = AssociatedElementIndex) then
      if (AssociatedElementKeywordIndex = -1) or (Expressions[i].AssociatedElementKeywordIndex = AssociatedElementKeywordIndex) then
        if (AssociatedExpressionKeywordIndex = -1) or (Expressions[i].AssociatedExpressionKeywordIndex = AssociatedExpressionKeywordIndex) then
          if (IncludeElementsTypes = []) or (Expressions[i].FRecognizedType in IncludeElementsTypes) then
            if not (Expressions[i].FRecognizedType in ExcludeElementsTypes) then
            begin
              Result := i;
              Break;
            end;
end;

procedure TcyDocER.LoadFromTesseractFile(const aFilename: String; const aPageNumber: Word{$IFDEF UNICODE}; const aEncoding: TEncoding{$ENDIF});
var
  FileLines: TStrings;
begin
  ClearPages;
  FileLines := TStringList.Create;
  FileLines.LoadFromFile(aFilename{$IFDEF UNICODE}, aEncoding{$ENDIF});
  LoadFromTesseractString(FileLines.Text, aPageNumber);
  FileLines.Free;
end;

procedure TcyDocER.LoadFromFile(const aFilename: String{$IFDEF UNICODE}; const aEncoding: TEncoding{$ENDIF});
var
  FileLines: TStrings;
begin
  FileLines := TStringList.Create;
  FileLines.LoadFromFile(aFilename{$IFDEF UNICODE}, aEncoding{$ENDIF});
  LoadFromStringList(FileLines);
  FileLines.Free;
end;

procedure TcyDocER.LoadFromStringList(aStrings: TStrings);
var
  MaxPages, l, LengthStrLine: Integer;
  StrLine: String;
  ExpressionString: String;
  ExpressionPageNumber: Integer;
  ExpressionOCRConfidence: Extended;
  ExpressionLeft, ExpressionTop, ExpressionRight, ExpressionBottom: Integer;
  PagePixelsWidth, PagePixelsHeight, PageResolution, PageTag, PageAngleRotation: Integer;
  PageTagStr: String;
const
  SeparatorCar = ';';

        // Reading from right !!!
        function CutLineValue: String;
        begin
          Result := '';
          while LengthStrLine > 0 do
            if StrLine[LengthStrLine] = SeparatorCar then
            begin
              Delete(StrLine, LengthStrLine, 1);
              Dec(LengthStrLine);
              Break;
            end
            else begin
              Result := StrLine[LengthStrLine] + Result;
              Delete(StrLine, LengthStrLine, 1);
              Dec(LengthStrLine);
            end;
        end;

begin
  if aStrings.Count = 0 then
    Exit;

  // *** Header *** //
  StrLine := aStrings[0];
  LengthStrLine := Length(StrLine);

  // On old versions, we have only 2 separators to separate PagePixelsHeight + SeparatorCar + PagePixelsWidth + SeparatorCar + PageResolution
  if SubString_Count(StrLine, SeparatorCar) > 2 then   // Detect new version with multipages definitions ...
  begin
    // 2015-11-12 Load definition for each page !
    while LengthStrLine <> 0 do
    begin
      PageTagStr := CutLineValue;
      if not TryStrToInt(CutLineValue, PageTag) then PageTag := 0;
      if not TryStrToInt(CutLineValue, PageAngleRotation) then PageAngleRotation := 0;

      if not TryStrToInt(CutLineValue, PagePixelsHeight) then PagePixelsHeight := 3508;
      if not TryStrToInt(CutLineValue, PagePixelsWidth) then PagePixelsWidth := 2480;
      if not TryStrToInt(CutLineValue, PageResolution) then PageResolution := 300;

      NewPage(PageResolution, PagePixelsWidth, PagePixelsHeight, PageAngleRotation, PageTag, PageTagStr);
    end;
  end
  else begin
    // Compatibility with old versions without multipage definitions :
    PageAngleRotation := 0;
    PageTag := 0;
    PageTagStr := '';
    if not TryStrToInt(CutLineValue, PagePixelsHeight) then PagePixelsHeight := 3508;
    if not TryStrToInt(CutLineValue, PagePixelsWidth) then PagePixelsWidth := 2480;
    if not TryStrToInt(CutLineValue, PageResolution) then PageResolution := 300;

    // 2016-04-06 :
    NewPage(PageResolution, PagePixelsWidth, PagePixelsHeight, PageAngleRotation, PageTag, PageTagStr);
  end;

  MaxPages := 0;

  // *** Expressions *** //
  for l := 1 to aStrings.Count-1 do
  begin
    StrLine := aStrings[l];
    LengthStrLine := Length(StrLine);

    ExpressionPageNumber := 0;
    ExpressionOCRConfidence := 0;
    ExpressionLeft := 0;
    ExpressionTop := 0;
    ExpressionRight := 0;
    ExpressionBottom := 0;

    try ExpressionBottom := StrToInt(CutLineValue); except end;
    try ExpressionRight := StrToInt(CutLineValue); except end;
    try ExpressionTop := StrToInt(CutLineValue); except end;
    try ExpressionLeft := StrToInt(CutLineValue); except end;
    ExpressionOCRConfidence := cyStrUtils.String_ToFloat(CutLineValue);
    try ExpressionPageNumber := StrToInt(CutLineValue); except end;
    ExpressionString := StrLine;  // No matter if Strline contains SeparatorCar !

    if ExpressionPageNumber > MaxPages then
      MaxPages := ExpressionPageNumber;

    if AddExpression(ExpressionString, ExpressionPageNumber, ExpressionOCRConfidence, classes.Rect(ExpressionLeft, ExpressionTop, ExpressionRight, ExpressionBottom)) <> -1 then
      ExpressionLoaded;        // Retrieve short/long date :
  end;

  // Add all pages for retro compatibility (we don' t save definition for each page previously) :
  while PageCount < MaxPages do
    NewPage(PageResolution, PagePixelsWidth, PagePixelsHeight);
end;

procedure TcyDocER.SaveToFile(const aFilename: String{$IFDEF UNICODE}; const aEncoding: TEncoding{$ENDIF});
var
  FileLines: TStrings;
begin
  FileLines := TStringList.Create;
  SaveToStringList(FileLines);
  FileLines.SaveToFile(aFilename{$IFDEF UNICODE}, aEncoding{$ENDIF});
  FileLines.Free;
end;

procedure TcyDocER.SaveToStringList(aStrings: TStrings);
var
  i: Integer;
  Str, StrLine: String;

const
  SeparatorCar = ';';

begin
  aStrings.Clear;

  // *** Header *** //
  // OLD ... aStrings.Add(intToStr(FResolution) + SeparatorCar + intToStr(FPixelsWidth) + SeparatorCar + intToStr(FPixelsHeight));

  // 2015-11-12 Save definition for each page !
  StrLine := '';
  for i := 0 to PageCount-1 do
  begin
    if i <> 0 then
      StrLine := StrLine + SeparatorCar;  // Page separator ...

    StrLine := StrLine + intToStr(Pages[i].FResolution) + SeparatorCar + intToStr(Pages[i].FPixelsWidth) + SeparatorCar + intToStr(Pages[i].FPixelsHeight)
        + SeparatorCar + intToStr(Pages[i].FAngleRotation) + SeparatorCar + intToStr(Pages[i].FTag) + SeparatorCar + Pages[i].FTagStr;
  end;

  aStrings.Add(StrLine);

  // *** Expressions *** //
  for i := 0 to ExpressionCount-1 do
  begin
    Str := Expressions[i].Value;

    StrLine := Str + SeparatorCar + intToStr(Expressions[i].FPageNumber) + SeparatorCar + FloatToStr(Expressions[i].FOCRConfidence) + SeparatorCar
                + intToStr(Expressions[i].FLeftPxPos) + SeparatorCar + intToStr(Expressions[i].FTopPxPos) + SeparatorCar
                 + intToStr(Expressions[i].FRightPxPos) + SeparatorCar + intToStr(Expressions[i].FBottomPxPos);
    aStrings.Add(StrLine);
  end;
end;

procedure TcyDocER.LoadFromTesseractString(aString: String; const aPageNumber: Integer);
var
  i, lengthString: Integer;
  StrExpression: String;
begin
  while PageCount < aPageNumber do
    NewPage(300, 2480, 3508);

  StrExpression := '';
  aString := aString + ' ';
  lengthString := length(aString);

  repeat
    aString := String_Subst(' %', '%', aString, csCaseSensitive, true);
  until pos(' %', aString) = 0;

  repeat
    aString := String_Subst(' ', '', aString, csCaseSensitive, true);
  until pos(' ', aString) = 0;

  // Add all words from file :
  for i := 1 to lengthString do
    if (aString[i] in [#$D, #$A, ' ']) or (i = lengthString) then
    begin
      AddExpression(StrExpression, aPageNumber, 0);
      StrExpression := '';
    end
    else
      StrExpression := StrExpression + aString[i];
end;

procedure TcyDocER.LoadFromTesseractBoxFile(const aFilename: String; {$IFDEF UNICODE} const aEncoding: TEncoding; {$ENDIF} const ImageDocumentPages, ImageResolution, ImagePixelsWidth, ImagePixelsHeight: Integer);
var
  FileLines: TStrings;
begin
  FileLines := TStringList.Create;
  FileLines.LoadFromFile(aFilename{$IFDEF UNICODE}, aEncoding{$ENDIF});
  LoadFromTesseractBoxStringList(FileLines, ImageDocumentPages, ImageResolution, ImagePixelsWidth, ImagePixelsHeight);
  FileLines.Free;
end;

// Convert Tesseract box file into Expressions :
procedure TcyDocER.LoadFromTesseractBoxStringList(aStrings: TStrings; const ImageDocumentPages, ImageResolution, ImagePixelsWidth, ImagePixelsHeight: Integer);
var
  l, LineCount: Integer;
  CarValue: String;
  CarPage, CarLeft, CarTop, CarRight, CarBottom: Integer;
  LineCars: Array of TOCRCarInfo;
  LineCarCount: Integer;
  MinLineTop, MaxLineBottom: Integer;

        // **************************************************************************************************************************

        procedure CutLineIntoExpressions;

                  // Handle small heights ("-" car) and small width ("i" car)
                  // If Height * 0.75 > Width ("i" char for exemple), we try to calculate correct font size
                  function CalcFontSize(CarIndex: Integer): Integer;
                  var MinSize, _width, _Height: Integer;
                  begin
                    MinSize := Round( (ImageResolution / 100) * 6 );    // Min font size recognition = 6
                    _Width  := LineCars[CarIndex].Right - LineCars[CarIndex].Left + 1;
                    _Height := LineCars[CarIndex].Bottom - LineCars[CarIndex].Top + 1;

                    if _Width < MinSize then
                      _Width := MinSize;

                    if _Height < MinSize then
                      _Height := MinSize;

                    if _Height * 0.75 > _Width then
                    begin
                      if _Height / _width <= 3
                      then
                        Result := Round((_Height * 0.9) * 0.8 + _Width * 0.2)  // H20/W7 = 20*0.9*0.8+7*0.2 = 15,8
                      else
                        if _Height / _Width <= 4.5
                        then Result := Round((_Height * 0.75) * 0.8 + _Width * 0.2)      // H25/W6 = 25*0.75*0.8+6*0.2 = 16,2
                        else Result := Round(_Height * 0.8);                             // H20/W4 = 20*0.8 = 16
                    end
                    else
                      if _Height > _Width
                      then Result := _Height
                      else Result := _Width;

                    if LineCars[CarIndex].Value = '@' then
                      Result := Round(Result * 0.6);
                  end;

                  // Space char is aproximatively 1/3 to 1/2 of font size
                  // Letters Spacing between cars is 15% font' s size
                  // Handle Courier new space char + anormal space between chars
                  // Handle when some word's caracters are embbeded into another ...
                  function CarBelongsToWord(CarIndex: Integer; LettersSpacing, NormalFontSize: Integer): Boolean;
                  var
                    MaxSpaceCharWidth: Integer;
                  begin
                    if NormalFontSize = 0 then
                    begin
                      Result := true;
                      Exit;
                    end;

                    Result := false;

                    // letters embedded into another (Logo or italic font):
                    if LettersSpacing < 0 then
                      LettersSpacing := 0;

                    if CalcFontSize(CarIndex) > NormalFontSize then
                      NormalFontSize := CalcFontSize(CarIndex);

                    // Handling spacing between "1" followed by another "1" (the normal char spacing is raised by 1/4 font px height):
                    if (CarIndex > 0) and (LineCars[CarIndex].Value[1] in ['l', '1']) then
                      if LineCars[CarIndex-1].Value[1] in ['l', '1'] then
                        Inc(LettersSpacing, (LineCars[CarIndex].Bottom - LineCars[CarIndex].Top) div 4);

                    // Handling anormal spacing between number and decimal separator for float/money values (exemple: " 3 8  ,  0 0   ") :
                    if (CarIndex > 0) and (LineCars[CarIndex].Value[1] in [',', '.']) then
                      if LineCars[CarIndex-1].Value[1] in ['0'..'9'] then
                        Inc(LettersSpacing, NormalFontSize div 2);    // Ok ...

                    // 0.25 = (normal char space width (33% font size) + Normal Spacing between cars (15% font size)) div 2 :
                    MaxSpaceCharWidth := Round(NormalFontSize * 0.261);  // 0.26 is not enougth for numbers recognition ("01/11/2011")

                    if LineCars[CarIndex].Left - LineCars[CarIndex-1].Right < LettersSpacing + MaxSpaceCharWidth then
                      Result := true;
                  end;

        var
          i, j, CharFontSize, LettersSpacing, CurrentWordNumber: Integer;
          aWord: String;
          aRect: TRect;
        begin
          if LineCarCount = 0 then Exit;

          if LineCarCount = 1 then
          begin
            AddExpression(LineCars[0].Value, LineCars[0].PageNumber, 0, Rect(LineCars[0].Left, LineCars[0].Top, LineCars[0].Right, LineCars[0].Bottom));
            Exit;
          end;

          // At least, we have 2 chars :
          CharFontSize := 0;
          CurrentWordNumber := 1;
          LettersSpacing := LineCars[LineCarCount-1].Right - LineCars[0].Left;

          // Detect words:
          for i := 0 to LineCarCount-1 do
          begin
            LineCars[i].WordNumber := CurrentWordNumber;

            j := i;
            // Will allow detect single char word followed by spacing! exemple: "tel : 000"  ...
            while CarBelongsToWord(j, LettersSpacing, CharFontSize) do
            begin
              // Smallest char spacing between chars from same word:
              if CharFontSize <> 0 then   // not first word char
                if LineCars[j].Left - LineCars[j-1].Right < LettersSpacing then
                  LettersSpacing := LineCars[j].Left - LineCars[j-1].Right;

              // Bigger char size :
              if CalcFontSize(j) > CharFontSize then
                CharFontSize := CalcFontSize(j);

              inc(j);
              if j > LineCarCount-1 then
                Break;
            end;

            // Detect if next char is in a new word:
            if i <> LineCarCount-1 then
            begin
              if not CarBelongsToWord(i+1, LettersSpacing, CharFontSize) then
              begin
                Inc(CurrentWordNumber);
                CharFontSize := 0;
                LettersSpacing := LineCars[LineCarCount-1].Right - LineCars[i+1].Left;
              end;
            end;
          end;

          // Create Expressions :
          aWord := '';
          aRect := Rect(-1, -1, -1, -1);
          CurrentWordNumber := 1;
          for i := 0 to LineCarCount-1 do
          begin
            if LineCars[i].WordNumber <> CurrentWordNumber then
            begin
              AddExpression(aWord, LineCars[i-1].PageNumber, 0, aRect);
              CurrentWordNumber := LineCars[i].WordNumber;
              aWord := '';
              aRect := Rect(-1, -1, -1, -1);
            end;

            // Build Word cars + Rect:
            aWord := aWord + LineCars[i].Value;
            if (LineCars[i].Left < aRect.Left) or (aRect.Left = -1) then aRect.Left := LineCars[i].Left;
            if (LineCars[i].Top < aRect.Top) or (aRect.Top = -1) then aRect.Top := LineCars[i].Top;
            if (LineCars[i].Right > aRect.Right) or (aRect.Right = -1) then aRect.Right := LineCars[i].Right;
            if (LineCars[i].Bottom > aRect.Bottom) or (aRect.Bottom = -1) then aRect.Bottom := LineCars[i].Bottom;
          end;

          // Add last word :
          AddExpression(aWord, LineCars[LineCarCount-1].PageNumber, 0, aRect);
        end;


        function CarInSameLine: Boolean;
        var y: Integer;
        begin
          Result := True;
          if LineCarCount = 0 then Exit;  // No cars

          Result := false;
          if LineCars[0].PageNumber <> CarPage then Exit;

          // See if char is just after last inserted in LineCars []:
          // Handle bricked Logo letters (like Tetris pieces)
          // '.' char followed by italic letter like 'f'can have same left position
          if CarRight > LineCars[LineCarCount-1].Right then
          // Not work if chars begins at same left position ... if CarLeft >= LineCars[LineCarCount-1].Right - (LineCars[LineCarCount-1].Right-LineCars[LineCarCount-1].Left) div 2 then
          begin
            // Check horizontal position :
            y := CarTop + (CarBottom - CarTop) div 2;
            if (y >= MinLineTop) and (y <= MaxLineBottom) then
              Result := true;
          end;
        end;

begin
  while PageCount < ImageDocumentPages do
    NewPage(ImageResolution, ImagePixelsWidth, ImagePixelsHeight);

  l := 0;
  LineCount := aStrings.Count;

  LineCarCount := 0;
  SetLength(LineCars, 0);
  MinLineTop := -1;
  MaxLineBottom := -1;

  while l <= LineCount-1 do    // All caracters ...
  begin
    ExtractTesseractBoxlineInfo(aStrings[l], CarValue, CarPage, CarLeft, CarTop, CarRight, CarBottom);

    if not CarInSameLine then
    begin
      // Cut into Expressions :
      CutLineIntoExpressions;

      LineCarCount := 0;
      SetLength(LineCars, 0);
      MinLineTop := -1;
      MaxLineBottom := -1;
    end;

    Inc(LineCarCount);
    SetLength(LineCars, LineCarCount);
    LineCars[LineCarCount-1].Value := CarValue;
    LineCars[LineCarCount-1].PageNumber := CarPage;
    LineCars[LineCarCount-1].Left := CarLeft;
    LineCars[LineCarCount-1].Top := CarTop;
    LineCars[LineCarCount-1].Right := CarRight;
    LineCars[LineCarCount-1].Bottom := CarBottom;
    LineCars[LineCarCount-1].WordNumber := 0;     // Not defined yet

    if (CarTop < MinLineTop) or (MinLineTop = -1) then
      MinLineTop := CarTop;

    if (CarBottom > MaxLineBottom) or (MaxLineBottom = -1) then
      MaxLineBottom := CarBottom;

    inc(l);
  end;

  // Cut last line into Expressions :
  CutLineIntoExpressions;
end;

function TcyDocER.FindExpression(const Value: Variant; const ValueType: TElementsType; const FromIndex, ToIndex: Integer): Integer;
var i: Integer;
begin
  Result := -1;

  try
    for i := FromIndex to ToIndex do
    begin
      case ValueType of
        etText, etExpressionKeyWord, etTextLine, etParagraph:
        begin
          if AnsiUpperCase(Expressions[i].FDERValue) = AnsiUpperCase(DoStringToDERCharSet(Value)) then
            Result := i;
        end;

        etNumbers:
        begin
          if Expressions[i].FRecognizedNumbers = Value then
            Result := i;
        end;

        etInteger:
        begin
          if Expressions[i].FRecognizedInteger <> '' then
            if Expressions[i].FRecognizedInteger = Value then
              Result := i;
        end;

        etFloat:
        begin
          if Expressions[i].FRecognizedFloat <> '' then
            if Expressions[i].FRecognizedFloat = Value then
              Result := i;
        end;

        etPercentage:
        begin
          if Expressions[i].FRecognizedFloat <> '' then
            if Expressions[i].FRecognizedFloat = Value then
              Result := i;
        end;

        etwebSite, etWebMail:
        begin
          if Expressions[i].FDERValue = Value then
            Result := i;
        end;

        etMoney:
        begin
          if Expressions[i].FRecognizedMoney <> '' then
            if Expressions[i].FRecognizedMoney = Value then
              Result := i;
        end;

        etDate:
        begin
          if Expressions[i].FRecognizedDate <> '' then
            if StrToInt(Expressions[i].FRecognizedDate) = Value then
              Result := i;
        end;
      end;

      if Result <> -1 then
        Break;
    end;
  except
  end;
end;

function TcyDocER.LocateExpression(Value: String; const FromIndex, ToIndex, MaxCarErrors: Integer; const Options: TLocateExpressionOptions; var RsltCarPos: Integer): Integer;
var
  i, LengthValue, LengthExpressionValue: Integer;
  ExpressionValue, UnmodifiedCarCaseValue: DERString;

      function RemovePointChar(const aDERString: DERString): DERString;
      begin
        Result := aDERString;
        String_SubstFast('.', '', Result);
      end;

      function GetValuePos: Integer;
      var CarErrors, c, s: Integer;
      begin
        Result := 0;

        if leSmartKeywordRec in Options then
        begin
          if ExpressionValue = '' then
          begin
            if LengthValue <= MaxCarErrors then
            begin
              Result := 1;
              RsltCarPos := Result;
            end;

            Exit;
          end;

          for c := 1 to (LengthExpressionValue - LengthValue)+1 do             // Search at possible positions
          begin
            CarErrors := 0;

            for s := 1 to LengthValue do
              if Value[s] <> ExpressionValue[c+s-1] then
              begin
                if not (UnmodifiedCarCaseValue[s] in ['a'..'z']) then  // Uppercase and symbols cars in Value must be the same as in expression
                  CarErrors := MaxCarErrors; // Let search at next c position ...

                Inc(CarErrors);
              end;

            if CarErrors <= MaxCarErrors then
            begin
              Result := c;
              RsltCarPos := Result;
              Exit;
            end;
          end;
        end
        else begin
          Result := pos(Value, ExpressionValue);
          RsltCarPos := Result;
        end;
      end;

begin
  Result := -1;

  Value  := DoStringToDERCharSet(Value);
  Value  := RemovePointChar(Value);
  UnmodifiedCarCaseValue := Value;
  LengthValue := Length(Value);

  if leInsensitive in Options then
    Value  := AnsiUpperCase(Value);

  for i := FromIndex to ToIndex do
  begin
    ExpressionValue := RemovePointChar(Expressions[i].DERValue);
    LengthExpressionValue := Length(ExpressionValue);

    if leInsensitive in Options then
      ExpressionValue := AnsiUpperCase(ExpressionValue);

    if lePartialKey in Options then
    begin
      // Must contain "value" at position 1 :
      if GetValuePos = 1 then
        Result := i;
    end
    else
      if leRelativePositionKey in Options then
      begin
        // In any position :
        if GetValuePos <> 0 then
          Result := i;
      end
      else begin
        // Must be equal :
        if LengthValue = LengthExpressionValue then
          if GetValuePos = 1 then
            Result := i;
      end;

    if Result <> -1 then
      Exit;
  end;
end;

function TcyDocER.ExpressionsInSameLine(const aExpressionIndex1, aExpressionIndex2: Integer): Boolean;
var
  y1: Integer;
begin
  // Calculate horizontal position :
  y1 := Expressions[aExpressionIndex1].FTopPxPos + (Expressions[aExpressionIndex1].FBottomPxPos - Expressions[aExpressionIndex1].FTopPxPos) div 2;
  Result := (y1 >= Expressions[aExpressionIndex2].FTopPxPos) and (y1 <= Expressions[aExpressionIndex2].FBottomPxPos);
end;

procedure TcyDocER.GetAroundExpressions(const aExpressionIndex, MaxCarErrors, ScopePx: Integer; const SearchValue: String; const SearchOptions: TLocateExpressionOptions; var RsltLeft, RsltTop, RsltRight, RsltBottom: Integer);

        function SearchValueMatch(aIndex: Integer): Boolean;
        var RsltCarPos: Integer;
        begin
          Result := true;
          if SearchValue = '' then Exit;

          Result := LocateExpression(SearchValue, aIndex, aIndex, MaxCarErrors, SearchOptions, RsltCarPos) = aIndex;
        end;

var
  i, x, LetterWidth, PosRsltLeft, PosRsltTop, PosRsltRight, PosRsltBottom: Integer;
begin
  RsltLeft := -1;    // Must be in same line
  RsltTop := -1;     // Must be in same column
  RsltRight := -1;   // Must be in same line
  RsltBottom := -1;  // Must be in same column

  PosRsltLeft := -1;
  PosRsltTop := -1;
  PosRsltRight := -1;
  PosRsltBottom := -1;

  if length(Expressions[aExpressionIndex].FValue) <> 0
  then LetterWidth := (Expressions[aExpressionIndex].FRightPxPos - Expressions[aExpressionIndex].FLeftPxPos) div length(Expressions[aExpressionIndex].FValue)
  else LetterWidth := 0;

  for i := 0 to ExpressionCount-1 do
  begin
    if (i = aExpressionIndex) or (Expressions[i].FPageNumber <> Expressions[aExpressionIndex].FPageNumber) then
      Continue;

    // Expression in same line ?
    if ExpressionsInSameLine(i, aExpressionIndex) then
    begin
      // Left :
      if Expressions[i].FLeftPxPos < Expressions[aExpressionIndex].FLeftPxPos then
        if Expressions[i].FRightPxPos + ScopePx >= Expressions[aExpressionIndex].FLeftPxPos then
          if (Expressions[i].FLeftPxPos > PosRsltLeft) or (PosRsltLeft = -1) then    // 24/04/2012
          begin
            if SearchValueMatch(i)
            then RsltLeft := i
            else RsltLeft := -1;   // 24/04/2012

            PosRsltLeft := Expressions[i].FLeftPxPos;
          end;

      // Right :
      if Expressions[i].FLeftPxPos > Expressions[aExpressionIndex].FLeftPxPos then  // Compare left again
        if Expressions[i].FLeftPxPos - ScopePx <=  Expressions[aExpressionIndex].FRightPxPos then
          if (Expressions[i].FLeftPxPos < PosRsltRight) or (PosRsltRight = -1) then   // 24/04/2012
          begin
            if SearchValueMatch(i)
            then RsltRight := i
            else RsltRight := -1;  // 24/04/2012

            PosRsltRight := Expressions[i].FLeftPxPos;
          end;
    end;

    // Expression in same "column" ?
    x := Expressions[i].FLeftPxPos + (Expressions[i].FRightPxPos - Expressions[i].FLeftPxPos) div 2;
    if (x >= Expressions[aExpressionIndex].FLeftPxPos) and (x <= Expressions[aExpressionIndex].FRightPxPos)
     or (abs(Expressions[i].FLeftPxPos - Expressions[aExpressionIndex].FLeftPxPos) < LetterWidth)
      or (abs(Expressions[i].FRightPxPos - Expressions[aExpressionIndex].FRightPxPos) < LetterWidth) then  // Solve problem if much more letters and text align = left or right
    begin
      // Top :
      if Expressions[i].FTopPxPos < Expressions[aExpressionIndex].FTopPxPos then
        if Expressions[i].FBottomPxPos + ScopePx >= Expressions[aExpressionIndex].FTopPxPos then
          if (Expressions[i].FTopPxPos > PosRsltTop) or (PosRsltTop = -1) then     // 24/04/2012
          begin
            if SearchValueMatch(i)
            then RsltTop := i
            else RsltTop := -1;  // 24/04/2012

            PosRsltTop := Expressions[i].FTopPxPos;
          end;

      // Bottom :
      if Expressions[i].FTopPxPos > Expressions[aExpressionIndex].FTopPxPos then  // Compare top again
        if Expressions[i].FTopPxPos - ScopePx <=  Expressions[aExpressionIndex].FBottomPxPos then
          if (Expressions[i].FTopPxPos < PosRsltBottom) or (PosRsltBottom = -1) then  // 24/04/2012
          begin
            if SearchValueMatch(i)
            then RsltBottom := i
            else RsltBottom := -1;  // 24/04/2012

            PosRsltBottom := Expressions[i].FTopPxPos;
          end;
    end;
  end;
end;

function TcyDocER.MergeExpressions(aExpressionList: TStrings): Integer;
var
  i, m, MergingExpressionIndex: Integer;
begin
  Result := -1;

  for i := 1 to aExpressionList.Count-1 do
  begin
    MergingExpressionIndex := StrToInt(aExpressionList[i]);
    Result := StrToInt(aExpressionList[0]);
    MergeExpressions(StrToInt(aExpressionList[i]), Result);

    // All expressions with index greater that MergingExpressionIndex moved up on Expression list !
    for m := 0 to aExpressionList.Count-1 do
      if StrToInt(aExpressionList[m]) > MergingExpressionIndex then
        aExpressionList[m] := intToStr( StrToInt(aExpressionList[m]) - 1 );
  end;
end;

procedure TcyDocER.MergeExpressions(aExpressionIndex, ToExpressionIndex: Integer; const SeparWithSpace: boolean = true);
var
  i: Integer;
  Separator: String;
begin
  if aExpressionIndex = ToExpressionIndex then Exit;

  if SeparWithSpace
  then Separator := ' '
  else Separator := '';

  Expressions[ToExpressionIndex].Value := Expressions[ToExpressionIndex].Value + Separator + Expressions[aExpressionIndex].Value;

  if Expressions[aExpressionIndex].FLeftPxPos < Expressions[ToExpressionIndex].FLeftPxPos then
    Expressions[ToExpressionIndex].FLeftPxPos := Expressions[aExpressionIndex].FLeftPxPos;

  if Expressions[aExpressionIndex].FTopPxPos < Expressions[ToExpressionIndex].FTopPxPos then
    Expressions[ToExpressionIndex].FTopPxPos := Expressions[aExpressionIndex].FTopPxPos;

  if Expressions[aExpressionIndex].FRightPxPos > Expressions[ToExpressionIndex].FRightPxPos then
    Expressions[ToExpressionIndex].FRightPxPos := Expressions[aExpressionIndex].FRightPxPos;

  if Expressions[aExpressionIndex].FBottomPxPos > Expressions[ToExpressionIndex].FBottomPxPos then
    Expressions[ToExpressionIndex].FBottomPxPos := Expressions[aExpressionIndex].FBottomPxPos;

  RecognizeExpressionType(ToExpressionIndex);

  // Delete aExpressionIndex from list :
  DeleteExpression(aExpressionIndex);

  // Resynch :
  if ToExpressionIndex > aExpressionIndex then
    Dec(ToExpressionIndex);

  for i := 0 to ExpressionCount-1 do
  begin
    if Expressions[i].FAssociatedExpressionKeywordIndex = aExpressionIndex then
      Expressions[i].FAssociatedExpressionKeywordIndex := ToExpressionIndex
    else
      if Expressions[i].FAssociatedExpressionKeywordIndex > aExpressionIndex then
        Expressions[i].FAssociatedExpressionKeywordIndex := Expressions[i].FAssociatedExpressionKeywordIndex - 1;
  end;

  // User Resynch :
  if Assigned(FOnExpressionMerged) then
    FOnExpressionMerged(Self, aExpressionIndex, toExpressionIndex);
end;

function TcyDocER.ExpressionsSideBySide(const ExpressionIndexAtLeft, ExpressionIndexAtRight, MaxPxSpacing: Integer): Boolean;
begin
  Result := false;
  if Expressions[ExpressionIndexAtLeft].FPageNumber <> Expressions[ExpressionIndexAtRight].FPageNumber then Exit;
  if Expressions[ExpressionIndexAtLeft].FLeftPxPos >= Expressions[ExpressionIndexAtRight].FLeftPxPos then Exit;

  // Check horizontal spacing between expressions:
  if Expressions[ExpressionIndexAtRight].FLeftPxPos - Expressions[ExpressionIndexAtLeft].FRightPxPos > MaxPxSpacing then Exit;

  // Check horizontal position (must intersect):
  if Expressions[ExpressionIndexAtLeft].FTopPxPos < Expressions[ExpressionIndexAtRight].FTopPxPos then
    Result := Expressions[ExpressionIndexAtLeft].FBottomPxPos >= Expressions[ExpressionIndexAtRight].FTopPxPos
  else
    Result := Expressions[ExpressionIndexAtLeft].FTopPxPos <= Expressions[ExpressionIndexAtRight].FBottomPxPos;
end;

// Returns located expression Index of the specicied keyword :
function TcyDocER.RecognizeElementKeyword(const ElementIndex, KeywordIndex: Integer; const InlineKeyword: Boolean): Boolean;
var
  ExpressionList: TStrings;
  DERWordStack: TStrings;
  RemainedExpression: DERString;
  aKeyword, CurrentWord: String;
  i, m, LocateCarPos, KeywordLength, ExpressionIndex, SearchWordIndex, AroundExpressionIndex, ScopePx, MergingExpressionIndex: Integer;
  LeftExpression, TopExpression, RightExpression, BottomExpression: Integer;
  SearchWords: Boolean;
  isExpressionAssigned: Boolean;
  MaxCarErrors, IncCarErrors: Integer;
  _AssociatedElementIndex, _AssociatedElementKeywordIndex: Integer;
  Locate1stWordOptions, LocateAroundOptions: TLocateExpressionOptions;

      // Cannot split in the middle of a word (Because the user's keyword could be just some first letters of a word) !
      procedure TrySplitOnRemainedExpression(aExpressionIndex: Integer);
      var
        LengthValue, LengthRemainedExpression, k, n: Integer;
        RemainedStr, NewValue: String;
      begin
        LengthValue := length(Expressions[aExpressionIndex].FValue);
        LengthRemainedExpression := Length(RemainedExpression);

        // for k := LengthValue downto 1 do  // 30/04/2012
        for k := LengthValue downto 2 do
        begin
          RemainedStr := Copy(Expressions[aExpressionIndex].FValue, k, LengthValue); // Copy from k
          // 05/05/2012 - Remove '.' char :
          String_SubstFast('.', '', RemainedStr);

          if length(DoStringToDERCharSet(RemainedStr)) = LengthRemainedExpression then
          begin
            // 30/04/2012  if not (RemainedStr[1] in ['.', 'a'..'z', 'A'..'Z', '0'..'9']) then
            if not (Expressions[aExpressionIndex].FValue[k-1] in ['.', 'a'..'z', 'A'..'Z', '0'..'9']) then  // Don' t split last word ("tot", "tel.", 'H.T.')
            begin
              NewValue := RemainedStr;

              // Modify actual expression:
              Expressions[aExpressionIndex].FValue := Copy(Expressions[aExpressionIndex].FValue, 1, k - 1);
              RecognizeExpressionType(aExpressionIndex);

              n := AddExpression(NewValue, Expressions[aExpressionIndex].FPageNumber, Expressions[aExpressionIndex].FOCRConfidence);

              if n <> -1 then
                with Expressions[n] do
                begin
                  FLeftPxPos := Expressions[aExpressionIndex].FLeftPxPos;
                  FTopPxPos := Expressions[aExpressionIndex].FTopPxPos;
                  FRightPxPos := Expressions[aExpressionIndex].FRightPxPos;
                  FBottomPxPos := Expressions[aExpressionIndex].FBottomPxPos;
                  // ExpressionAdded; Can' t call here because it is not inserted in reading words order ...
                end;
            end;

            Break;
          end;
        end;
      end;

begin
  Result := false;
  aKeyword := Elements[ElementIndex].KeyWords[KeywordIndex];
  if aKeyword = '' then Exit;

  if Assigned(FBeforeRecognizeElementKeyword) then
    FBeforeRecognizeElementKeyword(Self, ElementIndex, KeywordIndex);

  ExpressionList := TStringList.Create;
  if aKeyword[1] = ' '
  then Locate1stWordOptions := [lePartialKey, leInsensitive]  // Expression is "isolated" from a sentence or expression is the beginning of a sentence.
  else Locate1stWordOptions := [leRelativePositionKey, leInsensitive];
  LocateAroundOptions := [lePartialKey, leInsensitive];

  if roSmartKeywordRec in FRecognitionOptions then
  begin
    //Include(Locate1stWordOptions, leSmartKeywordRec);
    Locate1stWordOptions := Locate1stWordOptions + [leSmartKeywordRec];
    LocateAroundOptions := LocateAroundOptions + [leSmartKeywordRec];
  end;

  KeywordLength := Length(aKeyword);
  aKeyword := aKeyword + ' ';

  // *** Initialize DERWordStack with Keyword's word *** //
  IncCarErrors := 0;
  DERWordStack := TStringlist.Create;
  CurrentWord := '';
  for i := 1 to length(aKeyword) do
    if aKeyWord[i] <> ' ' then
    begin
      if aKeyWord[i] <> '.' then         // 05/05/2012  - points are ignored
        CurrentWord := CurrentWord + aKeyWord[i];
    end
    else
      if CurrentWord <> '' then
      begin
        CurrentWord := DoStringToDERCharSet(CurrentWord);
        if CurrentWord <> '' then
          DERWordStack.Add(CurrentWord);
        CurrentWord := '';
      end;

  // *** Search for DERWordStack items in List *** //
  if DERWordStack.Count > 0 then
  begin
    if DERWordStack.Count > 1 then
      IncCarErrors := 1;

    ExpressionIndex := 0;
    while (ExpressionIndex <> -1) and (not Result) do
    begin
      // Locate first aKeyword's word from "ExpressionIndex":
      MaxCarErrors := Length(DERWordStack[0]) div 6 + IncCarErrors;
      ExpressionIndex := LocateExpression(DERWordStack[0], ExpressionIndex, ExpressionCount-1, MaxCarErrors, Locate1stWordOptions, LocateCarPos);

      if ExpressionIndex <> -1 then
      begin
        // Handle when any element's Keyword text is part of another element's keyword :
        isExpressionAssigned := false;
        _AssociatedElementIndex := Expressions[ExpressionIndex].AssociatedElementIndex;
        _AssociatedElementKeywordIndex := Expressions[ExpressionIndex].AssociatedElementKeywordIndex;

        if (_AssociatedElementIndex = ElementIndex) and (_AssociatedElementKeywordIndex = KeywordIndex)
        then
          isExpressionAssigned := true   // Already assigned to this element and keyword ...
        else
          if Expressions[ExpressionIndex].RecognizedType <> etExpressionKeyWord then
            isExpressionAssigned := _AssociatedElementIndex <> -1
          else
            if (Expressions[ExpressionIndex].RecognizedType = etExpressionKeyWord) and (_AssociatedElementIndex <> -1) and (_AssociatedElementKeywordIndex <> -1) then
            begin
              if KeywordLength <= Length( Elements[_AssociatedElementIndex].KeyWords[_AssociatedElementKeywordIndex] ) then
                isExpressionAssigned := true;
            end;

        if not isExpressionAssigned then
        begin
          ExpressionList.Clear;
          ExpressionList.Add(intToStr(ExpressionIndex));
          RemainedExpression := AnsiUppercase(Expressions[ExpressionIndex].DERValue);
          // 05/05/2012 - Remove '.' char :
          String_SubstFast('.', '', RemainedExpression);
          // Remained Expression DERString :
          RemainedExpression := Copy(RemainedExpression, LocateCarPos + length(DERWordStack[0]), length(RemainedExpression));

          // *** Search for the others keyword words *** //
          SearchWords := True;
          SearchWordIndex := 1;    // Search for 2nd keyword's word
          AroundExpressionIndex := ExpressionIndex;

          while (SearchWords) and (SearchWordIndex <= DERWordStack.Count-1) do
          begin
            if RemainedExpression = '' then
            begin
              // Locate Expression Index with remained searched text:
              if InlineKeyword and (DERWordStack[SearchWordIndex] = ':')   // 21/04/2012 - Natural separator, can be far away from prior words ...
              then ScopePx := GetPagePixelsWidth(Expressions[AroundExpressionIndex].PageNumber)
              else ScopePx := (Expressions[AroundExpressionIndex].FBottomPxPos - Expressions[AroundExpressionIndex].FTopPxPos)* 2; // Twice of expression's height

              MaxCarErrors := Length(DERWordStack[SearchWordIndex]) div 6 + IncCarErrors;
              GetAroundExpressions(AroundExpressionIndex, MaxCarErrors, ScopePx, DERWordStack[SearchWordIndex], LocateAroundOptions, LeftExpression, TopExpression, RightExpression, BottomExpression);
              AroundExpressionIndex := RightExpression;

              if not InlineKeyword then
                if (BottomExpression <> -1) and (AroundExpressionIndex = -1) then
                  AroundExpressionIndex := BottomExpression;

              if AroundExpressionIndex <> -1 then
              begin
                ExpressionList.Add(intToStr(AroundExpressionIndex));
                RemainedExpression := AnsiUppercase(Expressions[AroundExpressionIndex].DERValue);
                // 05/05/2012 - Remove '.' char :
                // 2015-01-08 ... RemainedExpression := StringReplace(RemainedExpression, '.', '', [rfReplaceAll]);
                String_SubstFast('.', '', RemainedExpression);
              end
              else begin
                SearchWords := false;
                Break;
              end;
            end
            else
              LocateCarPos := pos(AnsiUppercase(DERWordStack[SearchWordIndex]), RemainedExpression);      // RHR - roSmartKeywordRec not handled here !

            if LocateCarPos = 1 then
            begin
              RemainedExpression := Copy(RemainedExpression, 1 + length(DERWordStack[SearchWordIndex]), length(RemainedExpression));
              Inc(SearchWordIndex);
            end
            else
              SearchWords := false;
          end;

          if SearchWords then  // All keyword's word retrieved:
            Result := true;
        end;

        Inc(ExpressionIndex);
      end;
    end;
  end;

  if Result then
    if Assigned(FOnValidateElementKeyword) then
      FOnValidateElementKeyword(Self, ElementIndex, KeywordIndex, ExpressionList, Result);

  if Result then
  begin
    // In some cases, Expression value contains element's keyword + element's value (exemple: "fax:06328974")
    if RemainedExpression <> '' then
    begin
      // Split expression in order to have the key in actual expression and the RemainedExpression in a new expression :
      TrySplitOnRemainedExpression(StrToInt(ExpressionList[ExpressionList.Count-1]));
    end;

    for i := 1 to ExpressionList.Count-1 do
    begin
      MergingExpressionIndex := StrToInt(ExpressionList[i]);
      MergeExpressions(StrToInt(ExpressionList[i]), StrToInt(ExpressionList[0]));

      // All expressions with index greater that MergingExpressionIndex moved up on Expression list !
      for m := 0 to ExpressionList.Count-1 do
        if StrToInt(ExpressionList[m]) > MergingExpressionIndex then
          ExpressionList[m] := intToStr( StrToInt(ExpressionList[m]) - 1 );
    end;

    Expressions[StrToInt(ExpressionList[0])].FAssociatedElementIndex := ElementIndex;
    Expressions[StrToInt(ExpressionList[0])].FAssociatedElementKeywordIndex := KeywordIndex;
    Expressions[StrToInt(ExpressionList[0])].FRecognizedType := etExpressionKeyWord;
  end;

  DERWordStack.Free;
  ExpressionList.Free;

  if Assigned(FAfterRecognizeElementKeyword) then
    FAfterRecognizeElementKeyword(Self, ElementIndex, KeywordIndex);
end;

// Recognize element's expressions that intersect with Rect :
function TcyDocER.RecognizeElementValuesFromRect(const OfElementIndex, OfElementKeywordIndex, OfExpressionKeywordIndex: Integer; const aValueType: TElementsType;
 const aPageNumber, LeftPos, TopPos, RightPos, BottomPos: Integer; const ValueLocation: TSearchValueLocation;
 ValueNumber, ValueCount: Integer; const PatternMode: Boolean; var UserAbort: Boolean): Integer;

       function ExpressionInArea(aExpressionIndex: Integer): Boolean;
       begin
         // Check horizontal intersection :
         if Expressions[aExpressionIndex].FLeftPxPos <= LeftPos
         then Result := Expressions[aExpressionIndex].FRightPxPos >= LeftPos
         else Result := Expressions[aExpressionIndex].FLeftPxPos <= RightPos;

         // Check vertical intersection :
         if Result then
           if Expressions[aExpressionIndex].FTopPxPos <= TopPos
           then Result := Expressions[aExpressionIndex].FBottomPxPos >= TopPos
           else Result := Expressions[aExpressionIndex].FTopPxPos <= BottomPos;
       end;

       function GetExpressionValue(aExpression: TOCRExpression; aValueType: TElementsType): String;
       begin
         Result := '';

         // etMoney can be recognized as etFloat:
         if (aValueType = etMoney) and (aExpression.FRecognizedType <> aValueType) then
           aValueType := etFloat;

         // etPercentage can be recognized as etFloat:
         if (aValueType = etPercentage) and (aExpression.FRecognizedType <> aValueType) then
           aValueType := etFloat;

          // etNumbers can be recognized as several etFloat :
         if (aValueType = etNumbers) and (aExpression.FRecognizedType <> aValueType) then
           aValueType := etFloat;

         // etFloat can be recognized as etInteger :
         if (aValueType = etFloat) and (aExpression.FRecognizedType <> aValueType) then
           aValueType := etInteger;

         if aValueType in [etText, etTextLine, etParagraph]  // Expression can be recognized as other type
         then
           Result := aExpression.FValue
         else
           if aExpression.FRecognizedType = aValueType then
             Result := aExpression.RecognizedValue;
       end;

var
  i, j, r, InsertPos, LastAddedExpressionIndex: Integer;
  ExpressionIndex0, ExpressionIndex1: Integer;
  ExpressionList: TStringList;
  Str, ExpressionValue: String;
  ValidationResult: TValidateElementValueResult;
  IncludeExpression, StopSearching, RemoveAll, MaskOk: Boolean;

       function GetInsertIndexPosition(aExpressionIndex: Integer): Integer;
       var
         i, ListExpressionIndex: Integer;
       begin
         Result := -1;

         for i := 0 to ExpressionList.Count -1 do
         begin
          ListExpressionIndex := StrToInt(ExpressionList[i]);

          case ValueLocation of
            slFromLeft:
            begin
              if ExpressionsInSameLine(aExpressionIndex, ListExpressionIndex) then
              begin
                if Expressions[aExpressionIndex].LeftPxPos < Expressions[aExpressionIndex].LeftPxPos then
                  Result := i;
              end
              else
                if Expressions[aExpressionIndex].TopPxPos < Expressions[aExpressionIndex].TopPxPos then
                  Result := i;
            end;

            slFromTop:
            begin
              if ExpressionsInSameLine(aExpressionIndex, ListExpressionIndex) then
              begin
                if Expressions[aExpressionIndex].LeftPxPos < Expressions[aExpressionIndex].LeftPxPos then
                  Result := i;
              end
              else
                if Expressions[aExpressionIndex].TopPxPos > Expressions[aExpressionIndex].TopPxPos then
                  Result := i;
            end;

            slFromRight:
            begin
              if ExpressionsInSameLine(aExpressionIndex, ListExpressionIndex) then
              begin
                if Expressions[aExpressionIndex].RightPxPos > Expressions[aExpressionIndex].RightPxPos then
                  Result := i;
              end
              else
                if Expressions[aExpressionIndex].TopPxPos < Expressions[aExpressionIndex].TopPxPos then
                  Result := i;
            end;

            slFromBottom:
            begin
              if ExpressionsInSameLine(aExpressionIndex, ListExpressionIndex) then
              begin
                if Expressions[aExpressionIndex].LeftPxPos < Expressions[aExpressionIndex].LeftPxPos then
                  Result := i;
              end
              else
                if Expressions[aExpressionIndex].TopPxPos < Expressions[aExpressionIndex].TopPxPos then
                  Result := i;
            end;
          end;

          if i <> -1 then
            Break;
         end;
       end;

begin
  Result := 0;
  UserAbort := false;
  ExpressionList := TStringList.Create;

  // Retrieve all expressions in defined area sorted as expected :
  for i := 0 to ExpressionCount-1 do
    if (Expressions[i].FPageNumber = aPageNumber) or (aPageNumber = 0) then
      if (not IsDERSymbols(Expressions[i].FDERValue)) or (ExpressionList.Count <> 0) then   // Remove symbol ...
        if ExpressionInArea(i) then
        begin
          IncludeExpression := true;

          if PatternMode then
          begin
            // 2013/10/29 Ignore expression from any already associated to another element :
            if Expressions[i].AssociatedElementIndex <> -1 then
              if Expressions[i].AssociatedElementIndex <> OfElementIndex then
                IncludeExpression := false;
          end
          else begin
            // 2015-09-30 Don' t include own keywords:
            if Expressions[i].AssociatedElementIndex = OfElementIndex then
              if Expressions[i].AssociatedElementKeywordIndex <> -1 then
                IncludeExpression := false;
          end;

          if IncludeExpression then
          begin
            // 2015-10-27 Insert position :
            InsertPos := GetInsertIndexPosition(i);

            if InsertPos = -1
            then ExpressionList.Add(intToStr(i))
            else ExpressionList.Insert(InsertPos, intToStr(i));
          end;
        end;


  if not PatternMode then
  begin
    // Remove all expressions from any already associated to an element until the end of the sorted list
    // Except if associated element type is date and searched value is Money/Float/Integer:
    for i := 0 to ExpressionList.Count-1 do
      with Expressions[ StrToInt(ExpressionList[i]) ] do
        if FAssociatedElementIndex <> -1 then
          if FAssociatedElementIndex <> OfElementIndex then                  // Found another Keyword or associated expression to another element ...
          begin
            RemoveAll := true;
            if FElements[OfElementIndex].ValueType in [etFloat, etMoney] then
              if FElements[FAssociatedElementIndex].ValueType = etDate then
                RemoveAll := false;

            if RemoveAll then
            begin
              for j := ExpressionList.Count-1 downto i do
                ExpressionList.Delete(j);

              Break;
            end;
          end;
  end;

  // Remove ValueNumber-1 or invalid expressions from list :
  while ExpressionList.Count <> 0 do
    if (GetExpressionValue(Expressions[ StrToInt(ExpressionList[0]) ], aValueType) = '') or (ValueNumber > 1) then // Value number for pattern recognition ...
    begin
      ExpressionList.Delete(0);
      Dec(ValueNumber);
    end
    else
      Break;

  // Validate candidates list :
  if Assigned(FOnRetrieveElementValuesFromRect) then
    FOnRetrieveElementValuesFromRect(Self, OfElementIndex, OfElementKeywordIndex, OfExpressionKeywordIndex, ExpressionList);

  // 2013-07-22 Merge expressions as needed if mask defined :
  if FElements[OfElementIndex].Mask <> '' then
  begin
    i := 0;
    Str := '';
    MaskOk := false;
    while i <= ExpressionList.Count-1 do
    begin
      Str := Str + Expressions[StrToInt(ExpressionList[i])].Value;

      if IsMatchMask2(FElements[OfElementIndex].Mask, Str) then
      begin
        MaskOk := true;
        Break;
      end
      else
        Inc(i);
    end;

    if MaskOk then
      for j := i downto 1 do
      begin
        ExpressionIndex0 := StrToInt(ExpressionList[j-1]);
        ExpressionIndex1 := StrToInt(ExpressionList[j]);

        MergeExpressions(ExpressionIndex1, ExpressionIndex0, false);
        ExpressionList.Delete(j);

        // Resynch:
        for r := 0 to ExpressionList.Count - 1 do
         if StrToInt(ExpressionList[r]) > ExpressionIndex1 then
           ExpressionList[r] := intToStr( StrToInt(ExpressionList[r]) - 1 );
      end;
  end;

  // 2013-07-22 Merge expressions if etTextLine or etParagraph :
  if aValueType in [etTextLine, etParagraph] then
  begin
    i := 0;
    while i <= ExpressionList.Count-2 do
    begin
      ExpressionIndex0 := StrToInt(ExpressionList[i]);
      ExpressionIndex1 := StrToInt(ExpressionList[i+1]);

      if (aValueType = etParagraph) or ExpressionsInSameLine(ExpressionIndex0, ExpressionIndex1) then
      begin
        if aValueType = etParagraph then
          if Expressions[ExpressionIndex0].BottomPxPos <= Expressions[ExpressionIndex1].TopPxPos then
          begin
            Expressions[ExpressionIndex0].Value := Expressions[ExpressionIndex0].Value + #13#10;
            RecognizeExpressionType(ExpressionIndex0);
          end;

        MergeExpressions(ExpressionIndex1, ExpressionIndex0);
        ExpressionList.Delete(i+1);

        // Resynch:
        for r := 0 to ExpressionList.Count - 1 do
         if StrToInt(ExpressionList[r]) > ExpressionIndex1 then
           ExpressionList[r] := intToStr( StrToInt(ExpressionList[r]) - 1 );
      end
      else
        if ValueCount = 0 // 2014-05-13
        then inc(i)       // 2014-05-13 - Try merge from next row
        else Break;
    end;
  end;

  if ExpressionList.Count <> 0 then
  begin
    // Assign Values to element and Keyword :
    StopSearching := false;
    LastAddedExpressionIndex := -1;
    while (ExpressionList.Count <> 0) and (not StopSearching) do
    begin
      if FElements[OfElementIndex].Mask <> '' then
      begin
        ExpressionValue := Expressions[ StrToInt(ExpressionList[0]) ].Value;
        MaskOk := IsMatchMask2(FElements[OfElementIndex].Mask, ExpressionValue);
      end
      else begin
        ExpressionValue := GetExpressionValue(Expressions[ StrToInt(ExpressionList[0]) ], aValueType);
        MaskOk := true;
      end;

      if MaskOk and (ExpressionValue <> '') then   // Mask ok and Same type ...
      begin
        ValidationResult := veValueOk;

        // Stop if distance is too big between current expression and LastAddedExpressionIndex :
        if (not PatternMode) and (LastAddedExpressionIndex <> -1) then
          if ValueLocation in [slFromTop, slFromBottom] then
            with Expressions[ StrToInt(ExpressionList[0]) ] do
              if Abs(FTopPxPos - Expressions[LastAddedExpressionIndex].FTopPxPos) > 3 * (FBottomPxPos - FTopPxPos) then  // + (Expressions[LastAddedExpressionIndex].FBottomPxPos - Expressions[LastAddedExpressionIndex].FTopPxPos)
              begin
                ValidationResult := veValueTooFar;
                StopSearching := true;
              end;

        // Validate event :
        if Assigned(FOnValidateElementValue) then
          FOnValidateElementValue(Self, OfElementIndex, OfElementKeywordIndex, OfExpressionKeywordIndex, StrToInt(ExpressionList[0]), ValidationResult);

        if ValidationResult <> veValueOk then
        begin
          ExpressionList.Delete(0);
          if ValidationResult = veInvalidValueStopSearching then
          begin
            UserAbort := true;
            StopSearching := True;
          end;

          // RHR AfterValidateEvent ...
          Continue;
        end;

        // Assign value to element:
        Inc(Result);
        LastAddedExpressionIndex := StrToInt(ExpressionList[0]);
        with Expressions[LastAddedExpressionIndex] do
        begin
          FAssociatedElementIndex := OfElementIndex;
          FAssociatedElementKeywordIndex := OfElementKeywordIndex;
          FAssociatedExpressionKeywordIndex := OfExpressionKeywordIndex;
        end;

        if aValueType in [etNumbers, etText] then
        begin
          if ValueLocation in [slFromTop, slFromBottom] then
          begin
            if ValueCount <> 0 then // Retrieve a specified number of expressions ...
              if ValueCount = Result then
                StopSearching := True;
          end
          else               // Retrieving from Left / right
            if ValueCount = 1 then
              StopSearching := True;
        end
        else
          if ValueCount = 0 then
          begin
            if (not PatternMode) and (ValueLocation = slFromLeft) then   // Retrieve only first value in the horizontal position ...
              StopSearching := True;
          end
          else           // Retrieve a specified number of expressions ...
            if ValueCount = Result then
              StopSearching := True;

      end
      else
        StopSearching := true;

      ExpressionList.Delete(0);
      // RHR AfterValidate event ...
    end;
  end;

  ExpressionList.Free;
end;

procedure TcyDocER.RecognizeElementValues(ElementIndex: Integer);
var
  ExpressionValuesCount, k, i, PageResolution: Integer;
  PatternExpressionIndex: Integer;
  ReadingDirection1: TReadingDirection;
  ReadingDirection2: TReadingDirection;
  ValueLocation: TSearchValueLocation;
  LeftPx, TopPx, RightPx, BottomPx, PageNumber: Integer;
  UserAborted: Boolean;

      procedure InitializeVariables(aReadingDirection: TReadingDirection; ExpressionIndex: Integer);
      var ExpressionHeight: Integer;
      begin
        case aReadingDirection of
          rdRight:     // We will search values at expression right side
          begin
            ValueLocation := slFromLeft;
            LeftPx   := Expressions[ExpressionIndex].FLeftPxPos;
            RightPx  := GetPagePixelsWidth(Expressions[ExpressionIndex].FPageNumber);

            // Using only HorzLinePx don' t work if keyword expression in more than single line and searched value not centered horinzontally :
            // HorzLinePx := Expressions[ExpressionIndex].FTopPxPos + ExpressionHeight div 2;
            ExpressionHeight := Expressions[ExpressionIndex].FBottomPxPos - Expressions[ExpressionIndex].FTopPxPos;
            TopPx    := Expressions[ExpressionIndex].FTopPxPos + ExpressionHeight div 4;
            BottomPx := Expressions[ExpressionIndex].FBottomPxPos - ExpressionHeight div 4;
          end;

          rdBottom:    // We will search values at expression bottom
          begin
            ValueLocation := slFromTop;
            LeftPx   := Expressions[ExpressionIndex].FLeftPxPos;   // We can have problems if values not aligned correctly RHR
            TopPx    := Expressions[ExpressionIndex].FTopPxPos;
            RightPx  := Expressions[ExpressionIndex].FRightPxPos;  // We can have problems if values not aligned correctly RHR
            BottomPx := GetPagePixelsHeight(Expressions[ExpressionIndex].FPageNumber);
          end;
        end;
      end;

begin
  if Elements[ElementIndex].PatternRecognition then
  begin
    if Elements[ElementIndex].FPatternPageNumberMode = ppFromBeginning
    then PageNumber := Elements[ElementIndex].FPatternPageNumber
    else PageNumber := PageCount - Elements[ElementIndex].FPatternPageNumber + 1;

    PageResolution := GetPageResolution(PageNumber);

    if Elements[ElementIndex].FPatternPositionMode = ppFromTopLeftPage then
    begin
      LeftPx   := MmToPx(Elements[ElementIndex].FPatternFromLeftMm, PageResolution);
      TopPx    := MmToPx(Elements[ElementIndex].FPatternFromTopMm, PageResolution);
      RightPx  := MmToPx(Elements[ElementIndex].FPatternToRightMm, PageResolution);
      BottomPx := MmToPx(Elements[ElementIndex].FPatternToBottomMm, PageResolution);
    end
    else begin
      // Get associated keyword:
      PatternExpressionIndex := -1;
      for i := 0 to ExpressionCount-1 do
        if Expressions[i].FAssociatedElementIndex = ElementIndex then
        begin
          PatternExpressionIndex := i;
          Break;
        end;

      if PatternExpressionIndex = -1 then Exit;

      LeftPx   := MmToPx(Elements[ElementIndex].FPatternFromLeftMm + Expressions[PatternExpressionIndex].FLeftPxPos, PageResolution);
      TopPx    := MmToPx(Elements[ElementIndex].FPatternFromTopMm + Expressions[PatternExpressionIndex].FTopPxPos, PageResolution);
      RightPx  := MmToPx(Elements[ElementIndex].FPatternToRightMm + Expressions[PatternExpressionIndex].FRightPxPos, PageResolution);
      BottomPx := MmToPx(Elements[ElementIndex].FPatternToBottomMm + Expressions[PatternExpressionIndex].FBottomPxPos, PageResolution);
    end;

    RecognizeElementValuesFromRect(
                           ElementIndex,
                           -1,
                           -1,
                           Elements[ElementIndex].FValueType,
                           PageNumber,
                           LeftPx,
                           TopPx,
                           RightPx,
                           BottomPx,
                           Elements[ElementIndex].FPatternValueLocation,
                           Elements[ElementIndex].FPatternValueNumber,
                           Elements[ElementIndex].FValueCount,
                           True, UserAborted);

    Exit;
  end;


  if Elements[ElementIndex].ValueCount = 1 then
  begin
    ReadingDirection1 := rdRight;
    ReadingDirection2 := rdBottom;
  end
  else begin
    ReadingDirection1 := rdBottom;
    ReadingDirection2 := rdRight;
  end;

  // Deal keywords list by roKeywordsByPriority:
  for k := 0 to Elements[ElementIndex].KeyWords.Count-1 do
  begin
    i := 0;
    while i <= ExpressionCount-1 do
    begin
      if Expressions[i].FRecognizedType = etExpressionKeyWord then
        if (Expressions[i].AssociatedElementIndex = ElementIndex) and (Expressions[i].AssociatedElementKeywordIndex = k) then
        begin
          // Read associated value(s) from ReadingDirection1 :
          InitializeVariables(ReadingDirection1, i);

          ExpressionValuesCount := RecognizeElementValuesFromRect(
                                     ElementIndex,
                                     k,
                                     i,
                                     Elements[ElementIndex].FValueType,
                                     Expressions[i].FPageNumber,
                                     LeftPx,
                                     TopPx,
                                     RightPx,
                                     BottomPx,
                                     ValueLocation,
                                     1,
                                     Elements[ElementIndex].FValueCount, false, UserAborted);

          // Read associated value(s) from ReadingDirection2 :
          if (ExpressionValuesCount = 0) and (not UserAborted) then
          begin
            InitializeVariables(ReadingDirection2, i);
            RecognizeElementValuesFromRect(
                                       ElementIndex,
                                       k,
                                       i,
                                       Elements[ElementIndex].FValueType,
                                       Expressions[i].FPageNumber,
                                       LeftPx,
                                       TopPx,
                                       RightPx,
                                       BottomPx,
                                       ValueLocation,
                                       1,
                                       Elements[ElementIndex].FValueCount, false, UserAborted);
          end;
        end;

      Inc(i);
    end;
  end;
end;

procedure TcyDocER.InitializeRecognition;
var
  i, m: Integer;
  MoneyHandled: Boolean;
begin
  // Initialize :
  for i := 0 to ExpressionCount-1 do
  begin
    Expressions[i].FAssociatedElementIndex := -1;
    Expressions[i].FAssociatedElementKeywordIndex := -1;
    Expressions[i].FAssociatedExpressionKeywordIndex := -1;
    if Expressions[i].FRecognizedType = etExpressionKeyWord then
      Expressions[i].FRecognizedType := etText;
  end;

  // Merge some separated cars in some cases :
  for i := ExpressionCount-1 downto 0 do
  begin
    // Merge percentage symbol with its value :
    if Expressions[i].FDERValue = '%' then
      if i > 0 then
        if Expressions[i-1].FRecognizedType in [etInteger, etFloat] then
          if ExpressionsSideBySide(i-1, i, Round((Expressions[i].FBottomPxPos - Expressions[i].FTopPxPos) * AutoMergeExpressionsRatio)) then
            MergeExpressions(i, i-1);

    // Merge money char with its value:
    for m := 1 to length(DERMoneyCars) do
      if Expressions[i].FDERValue = DERMoneyCars[m] then
      begin
        MoneyHandled := false;

        // Value is before ?
        if i > 0 then
          if Expressions[i-1].FRecognizedType in [etInteger, etFloat] then
            if ExpressionsSideBySide(i-1, i, Round((Expressions[i].FBottomPxPos - Expressions[i].FTopPxPos) * AutoMergeExpressionsRatio)) then
            begin
              MoneyHandled := true;
              MergeExpressions(i, i-1);
            end;

        // Value is after ?
        if not MoneyHandled then
          if i < ExpressionCount-1 then
            if Expressions[i+1].FRecognizedType in [etInteger, etFloat] then
              if ExpressionsSideBySide(i, i+1, Round((Expressions[i].FBottomPxPos - Expressions[i].FTopPxPos) * AutoMergeExpressionsRatio)) then
                MergeExpressions(i, i+1);
      end;
  end;
end;

function TcyDocER.IsElementKeyword(const aElementIndex: Integer; aStr: String): Integer;
var
  k: Integer;
  aKeyword: String;
begin
  Result := -1;
  aStr := AnsiUpperCase( DoStringToDERCharSet(aStr) );

  for k := 0 to Elements[aElementIndex].KeyWords.Count-1 do
  begin
    aKeyword := AnsiUpperCase( DoStringToDERCharSet(Elements[aElementIndex].KeyWords[k]) );

    if aKeyword = aStr then
    begin
      Result := k;
      Break;
    end;
  end;
end;

procedure TcyDocER.RecognizeElementsKeywords;
type
  rKeywords = record
    ElementIndex: Integer;
    KeywordIndex: Integer;
    LengthKeyword: Integer;
  end;

          procedure OrderByLengthAndRecognizeKeywords;
          var
            SortedKeywords: Array of rKeywords;
            NewElementIndex, NewKeywordIndex, NewLengthKeyword: Integer;
            TmpElementIndex, TmpKeywordIndex, TmpLengthKeyword: Integer;
            i, s, k, LengthSortedKeywords: Integer;
            ExpressionFound: Boolean;
          begin
            LengthSortedKeywords := 0;

            for i := 0 to FElements.Count-1 do
              for k := 0 to FElements[i].FKeyWords.Count-1 do
              begin
                NewElementIndex  := i;
                NewKeywordIndex  := k;
                NewLengthKeyword := Length(FElements[i].FKeyWords[k]);

                // Insert in array ordered by keyword's length :
                for s := 0 to LengthSortedKeywords-1 do
                  if NewLengthKeyword > SortedKeywords[s].LengthKeyword then
                  begin
                    TmpElementIndex  := SortedKeywords[s].ElementIndex;
                    TmpKeywordIndex  := SortedKeywords[s].KeywordIndex;
                    TmpLengthKeyword := SortedKeywords[s].LengthKeyword;

                    SortedKeywords[s].ElementIndex  := NewElementIndex;
                    SortedKeywords[s].KeywordIndex  := NewKeywordIndex;
                    SortedKeywords[s].LengthKeyword := NewLengthKeyword;

                    NewElementIndex  := TmpElementIndex;
                    NewKeywordIndex  := TmpKeywordIndex;
                    NewLengthKeyword := TmpLengthKeyword;
                  end;

                // Insert at the end :
                Inc(LengthSortedKeywords);
                SetLength(SortedKeywords, LengthSortedKeywords);
                SortedKeywords[LengthSortedKeywords-1].ElementIndex  := NewElementIndex;
                SortedKeywords[LengthSortedKeywords-1].KeywordIndex  := NewKeywordIndex;
                SortedKeywords[LengthSortedKeywords-1].LengthKeyword := NewLengthKeyword;
              end;

            // Recognize inline (in a single line) keywords :
            if FRecognitionPriorityMode = rpSinglelineKeywordsLength then
              for k := 0 to LengthSortedKeywords-1 do
                repeat
                  ExpressionFound := RecognizeElementKeyword(SortedKeywords[k].ElementIndex, SortedKeywords[k].KeywordIndex, true);   // Search same keyword until not ExpressionFound
                until not ExpressionFound;

            // Recognize keywords :
            for k := 0 to LengthSortedKeywords-1 do
              repeat
                ExpressionFound := RecognizeElementKeyword(SortedKeywords[k].ElementIndex, SortedKeywords[k].KeywordIndex, false);   // Search same keyword until not ExpressionFound
              until not ExpressionFound;
          end;

          // Find all element' s Keyword ocurrences in the document:
          procedure RecognizeElementKeywords(ElementIndex: Integer);
          var
            ExpressionFound: Boolean;
            k: Integer;
          begin
            // Search for all keywords of specified element:
            for k := 0 to FElements[ElementIndex].FKeyWords.Count-1 do
              repeat
                ExpressionFound := RecognizeElementKeyword(ElementIndex, k, false);   // Search same keyword until not ExpressionFound
              until not ExpressionFound;
          end;

var
  i: Integer;
begin
  if Assigned(FBeforeRecognizeElementsKeywords) then
    FBeforeRecognizeElementsKeywords(Self);

  // *** Find Keywords *** //
  if roKeywordsByPriority in FRecognitionOptions then
  begin
    OrderByLengthAndRecognizeKeywords;
  end
  else begin
    for i := 0 to Elements.Count-1 do
      RecognizeElementKeywords(i);
  end;
end;

procedure TcyDocER.RecognizeElementsValues;
var
  i: Integer;
begin
  if Assigned(FBeforeRecognizeElementsValues) then
    FBeforeRecognizeElementsValues(Self);

  // *** Find values *** //
  for i := 0 to Elements.Count-1 do       // Single value recognition ...
    if Elements[i].ValueCount = 1 then
      RecognizeElementValues(i);

  for i := 0 to Elements.Count-1 do       // Multiple values recognition ...
    if Elements[i].ValueCount <> 1 then
      RecognizeElementValues(i);
end;

function TcyDocER.RecognizeMonth(aDERString: DERString): Integer;
var
  m, i: Integer;
  ShortMonthValue: DERString;
begin
  Result := 0;
  if FShortMonthNames.Count < 12 then Exit;

  aDERString := AnsiUppercase(aDERString);

  for m := 1 to 12 do
  begin
    ShortMonthValue := AnsiUppercase( DoStringToDERCharSet(FShortMonthNames[m-1]) );

    for i := 1 to length(ShortMonthValue) do
      if not (ShortMonthValue[i] in ['A'..'Z']) then  // Remove any invalid char like '.' from "Jan." for exemple ...
      begin
        Delete(ShortMonthValue, i, Length(ShortMonthValue));
        Break;
      end;

    if pos(ShortMonthValue, aDERString) > 0 then
    begin
      Result := m;
      Break;
    end;
  end;
end;

procedure TcyDocER.RecognizeElements;
begin
  InitializeRecognition;

  RecognizeElementsKeywords;
  RecognizeElementsValues;
end;

// Handle some anormal situations :
// - 3 625,00 divided into 2 expressions ("3" and "625,00")
// - 3 625,00  divided into 2 expressions ("3" and "625,00")
// - 3 625 divided into 2 expressions ("3" and "625")
// - 3 625  divided into 2 expressions ("3" and "625")
// - Long date detection ([day] + [Month name] + [year]) :
procedure TcyDocER.ExpressionAdded;
var
  b: Boolean;
  pxTolerence1, pxTolerence2, pxTolerence, aYear, aMonth, aDay: Integer;
  aDate: TDateTime;

  DateSeparator: String;
  AnalyzeExpression: Integer;

      // See there' s natural separators into expression value: '()'
      function CanMergeExpressions(aExpressionIndex1, aExpressionIndex2: Integer): Boolean;
      begin
        Result := true;
        if pos('(', Expressions[aExpressionIndex1].Value) <> 0 then Result := false;
        if pos(')', Expressions[aExpressionIndex1].Value) <> 0 then Result := false;

        if pos('(', Expressions[aExpressionIndex2].Value) <> 0 then Result := false;
        if pos(')', Expressions[aExpressionIndex2].Value) <> 0 then Result := false;
      end;

begin
  if ExpressionCount <= 1 then Exit;

  // Added on 2013-07-19 :
  pxTolerence1 := Round((Expressions[ExpressionCount-1].FBottomPxPos - Expressions[ExpressionCount-1].FTopPxPos) * AutoMergeExpressionsRatio);
  pxTolerence2 := Round((Expressions[ExpressionCount-2].FBottomPxPos - Expressions[ExpressionCount-2].FTopPxPos) * AutoMergeExpressionsRatio);

  if pxTolerence1 > pxTolerence2
  then pxTolerence := pxTolerence1
  else pxTolerence := pxTolerence2;

  // Merge with negative symbol :
  if Expressions[ExpressionCount-2].FDERValue = '-' then
    if Expressions[ExpressionCount-1].FRecognizedType in [etMoney, etFloat, etInteger, etPercentage] then
      if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then        // Added on 2014/01/02 ...
      begin
        if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
        begin
          MergeExpressions(ExpressionCount-1, ExpressionCount-2);
          if ExpressionCount <= 1 then Exit;
        end;
      end;

  // Short/long month date detection ("15" + "january" + "1977" or "15" + "jan" + "1977") :
  if FShortMonthNames.Count >= 12 then
    if (ExpressionCount > 2) and (Expressions[ExpressionCount-1].FRecognizedType = etInteger) then
      if Length(Expressions[ExpressionCount-1].FRecognizedInteger) = 4 then            // Can use StrToInt() ?
      if (StrToInt(Expressions[ExpressionCount-1].FRecognizedInteger) > Recognition_MinDate)          // We are in presence of a year!?
         and (StrToInt(Expressions[ExpressionCount-1].FRecognizedInteger) < Recognition_MaxDate) then
        if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
        begin
          aMonth := RecognizeMonth(Expressions[ExpressionCount-2].FDERValue);

          if aMonth <> 0 then
            if ExpressionsSideBySide(ExpressionCount-3, ExpressionCount-2, pxTolerence) then
              if Expressions[ExpressionCount-3].FRecognizedType = etInteger then
                if length(Expressions[ExpressionCount-3].FRecognizedInteger) < 3 then // We are in presence of a day !?
                  if StrToInt(Expressions[ExpressionCount-3].FRecognizedInteger) > 0 then
                    try
                      // Validate date :
                      aYear := StrToInt(Expressions[ExpressionCount-1].FRecognizedInteger);
                      aDay  := StrToInt(Expressions[ExpressionCount-3].FRecognizedInteger);

                      if (aDay > 0) and (aDay < 32) then
                        if TryToEncodeDate(aYear, aMonth, aDay, aDate) then
                        begin
                          // Merge 3 expressions into a date :
                          MergeExpressions(ExpressionCount-1, ExpressionCount-2);
                          MergeExpressions(ExpressionCount-1, ExpressionCount-2);  // Carefull: ExpressionCount was modified by MergeExpressions() call ...

                          with Expressions[ExpressionCount-1] do
                          begin
                            FRecognizedDate := intToStr(Trunc(aDate));
                            FRecognizedType := etDate;
                          end;
                        end;
                    except

                    end;
        end;

  // dd/mm/yy(yy) date format detection on several expressions ("15" + "/" + "01" + "/" + "1977") :   // Added on 19/05/2013
  if Expressions[ExpressionCount-1].FRecognizedType = etInteger then
  begin
    aYear := 0;
    if Length(Expressions[ExpressionCount-1].FRecognizedInteger) in [2, 4] then
      aYear := StrToInt(Expressions[ExpressionCount-1].FRecognizedInteger);

    if Length(Expressions[ExpressionCount-1].FRecognizedInteger) = 2 then
      aYear := aYear + 2000;

    if (aYear > 1940) and (aYear < 2100) then
    begin
      aDay := 0;
      aMonth := 0;
      DateSeparator := '';
      AnalyzeExpression := (ExpressionCount-1) - 1;

      // Search month :
      while (AnalyzeExpression >= 0) and (aMonth = 0) do
      begin
        if not ExpressionsSideBySide(AnalyzeExpression, AnalyzeExpression+1, pxTolerence) then
          aMonth := -1
        else
          if Expressions[AnalyzeExpression].FRecognizedType <> etInteger then
          begin
            if (Expressions[AnalyzeExpression].FDERValue = '/') or (Expressions[AnalyzeExpression].FDERValue = '-') or (Expressions[AnalyzeExpression].FDERValue = '.')
            then DateSeparator := Expressions[AnalyzeExpression].FDERValue
            else aMonth := -1;
          end
          else
            if not TryStrToInt(Expressions[AnalyzeExpression].FRecognizedInteger, aMonth) then
              aMonth := -1;

        dec(AnalyzeExpression);
      end;

      if (aMonth > 0) and (aMonth < 13) then
      begin
        while (AnalyzeExpression >= 0) and (aDay = 0) do
        begin
          if not ExpressionsSideBySide(AnalyzeExpression, AnalyzeExpression+1, pxTolerence) then
            aDay := -1
          else
            if Expressions[AnalyzeExpression].FRecognizedType <> etInteger then
            begin
              if (Expressions[AnalyzeExpression].FDERValue = '/') or (Expressions[AnalyzeExpression].FDERValue = '-') or (Expressions[AnalyzeExpression].FDERValue = '.') then
              begin
                if DateSeparator <> Expressions[AnalyzeExpression].FDERValue then
                  aDay := -1;
              end
              else
                aDay := -1;
            end
            else
              if not TryStrToInt(Expressions[AnalyzeExpression].FRecognizedInteger, aDay) then
                aDay := -1;

          dec(AnalyzeExpression);
        end;
      end;

      if (aDay > 0) and (aDay < 32) then
        try
          if TryToEncodeDate(aYear, aMonth, aDay, aDate) then
          begin
            // Merge all expressions into a date :
            while ExpressionCount-1 > AnalyzeExpression+1 do
              MergeExpressions(ExpressionCount-1, ExpressionCount-2);  // Info: ExpressionCount was modified on MergeExpressions() ...

            with Expressions[ExpressionCount-1] do
            begin
              FRecognizedDate := intToStr(Trunc(aDate));
              FRecognizedType := etDate;
            end;
          end;
        except
        end;
    end;
  end;

  // Merge with prior added expression value for cutted float/integer/money/percentage values:
  case Expressions[ExpressionCount-1].FRecognizedType of
    etMoney:
    begin
      if ExpressionCount > 1 then
        if Expressions[ExpressionCount-1].FDERValue[1] in ['.', '0'..'9'] then    // Merge if money symbol at the end!
        begin
          if Expressions[ExpressionCount-1].FRecognizedFloat = Expressions[ExpressionCount-1].FRecognizedInteger
          then b := Expressions[ExpressionCount-2].FRecognizedType in [etFloat, etInteger]                         // Money value is an integer
          else b := Expressions[ExpressionCount-2].FRecognizedType in [etInteger];                                 // Money value is a float

          if b then
            if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
              if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
              begin
                MergeExpressions(ExpressionCount-1, ExpressionCount-2);
              end;
        end;
    end;

    etFloat:                //  "22  33,6"    or    "22 33,6"     // Don' t merge if  "2015 11:06"  !!!
    begin
      if ExpressionCount > 1 then
        case Expressions[ExpressionCount-2].FRecognizedType of
          etInteger:
            // 2016-03-31 Decimal separator on RecognzedFloat is cyDERUtils.LocalFormatSettings.DecimalSeparator ...
            // if pos(',', Expressions[ExpressionCount-1].RecognizedFloat) in [4,7,10] then
            if pos(cyDERUtils.LocalFormatSettings.DecimalSeparator, Expressions[ExpressionCount-1].RecognizedFloat) in [4,7,10] then    // merge '2015' with '000.09' but don' t merge  '2015' with '11:06'
            // if (StrToInt(Expressions[ExpressionCount-2].RecognizedInteger) < Recognition_MinDate) or (StrToInt(Expressions[ExpressionCount-2].RecognizedInteger) > Recognition_MaxDate) then
              if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
                if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
                begin
                  MergeExpressions(ExpressionCount-1, ExpressionCount-2);
                end;

          etMoney:
            if not (Expressions[ExpressionCount-2].FDERValue[1] in ['.', '0'..'9']) then    // Merge if money symbol at the beginning!
              if Expressions[ExpressionCount-2].FRecognizedFloat = Expressions[ExpressionCount-2].FRecognizedInteger then    // Money value is an integer
                if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
                  if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
                  begin
                    MergeExpressions(ExpressionCount-1, ExpressionCount-2);
                  end;
        end;
    end;

    etInteger, etPercentage:
    begin
      if ExpressionCount > 1 then
       if (Expressions[ExpressionCount-1].FRecognizedType <> etPercentage) or (Expressions[ExpressionCount-2].FRecognizedType <> etMoney) then // Cannot merge money with percentage expressions
        case Expressions[ExpressionCount-2].FRecognizedType of
          etText:
            if (ExpressionCount > 2) and (Expressions[ExpressionCount-2].FDERValue = '.') then // Decimal separator
              if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
                if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
                  if CanMergeExpressions(ExpressionCount-2, ExpressionCount-3) then
                    case Expressions[ExpressionCount-3].FRecognizedType of
                      etInteger:
                        if ExpressionsSideBySide(ExpressionCount-3, ExpressionCount-2, pxTolerence) then
                        begin
                          MergeExpressions(ExpressionCount-1, ExpressionCount-2);
                          MergeExpressions(ExpressionCount-1, ExpressionCount-2);
                        end;

                      etMoney:
                        if not (Expressions[ExpressionCount-3].FDERValue[1] in ['0'..'9']) then    // Merge if money symbol at the beginning!
                          if ExpressionsSideBySide(ExpressionCount-3, ExpressionCount-2, pxTolerence) then
                          begin
                            MergeExpressions(ExpressionCount-1, ExpressionCount-2);
                            MergeExpressions(ExpressionCount-1, ExpressionCount-2);
                          end;
                    end;

          etInteger:
            if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
              if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
                MergeExpressions(ExpressionCount-1, ExpressionCount-2);

          etFloat:
            if Expressions[ExpressionCount-1].RecognizedFloat = Expressions[ExpressionCount-1].RecognizedInteger then
              if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
                if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
                  MergeExpressions(ExpressionCount-1, ExpressionCount-2);

          etMoney:
            if not (Expressions[ExpressionCount-2].FDERValue[1] in ['.', '0'..'9']) then    // Merge if money symbol at the beginning!
              if ExpressionsSideBySide(ExpressionCount-2, ExpressionCount-1, pxTolerence) then
                if CanMergeExpressions(ExpressionCount-1, ExpressionCount-2) then
                  MergeExpressions(ExpressionCount-1, ExpressionCount-2);
        end;
    end;
  end;
end;

procedure TcyDocER.ExpressionLoaded;
var
  aYear, aMonth, aDay: Integer;
  aDate: TDateTime;
begin
  if ExpressionCount = 0 then Exit;  // 2016-06-20 ...

  // short/long Date detection :
  if FShortMonthNames.Count >= 12 then
    if SubString_Count(Expressions[ExpressionCount-1].FValue, ' ') >= 3 then
      if TryStrToInt( SubString_Get(Expressions[ExpressionCount-1].FValue, ' ', 3 ), aYear ) then
        if (aYear > 1940) and (aYear < 2100) then
        begin
          aMonth := RecognizeMonth( SubString_Get(Expressions[ExpressionCount-1].FValue, ' ', 2 ) );

          if aMonth <> 0 then
            if TryStrToInt( SubString_Get(Expressions[ExpressionCount-1].FValue, ' ', 1 ), aDay ) then
              if (aDay > 0) and (aDay < 32) then
                try
                  if TryToEncodeDate(aYear, aMonth, aDay, aDate) then
                    with Expressions[ExpressionCount-1] do
                    begin
                      FRecognizedDate := intToStr(Trunc(aDate));
                      FRecognizedType := etDate;
                    end;
                except

                end;
        end;

end;

function TcyDocER.ExpressionInRect(const aExpressionIndex: Integer; const InRect: TRect; const AllowPartiallyInside: Boolean = true): Boolean;
var DestRect: TRect;
begin
  Result := false;
  with Expressions[aExpressionIndex] do
    if AllowPartiallyInside then
      Result := IntersectRect(DestRect, InRect, classes.Rect(FLeftPxPos, FTopPxPos, FRightPxPos, FBottomPxPos))
    else
      if (FLeftPxPos >= InRect.Left) and (FTopPxPos >= InRect.Top) and (FRightPxPos <= InRect.Right) and (FBottomPxPos <= InRect.Bottom) then
        Result := true;
end;

// - Long date detection ([day] + [Month name] + [year]) :
procedure TcyDocER.RecognizeLongDates;
var
  i, pxTolerence, aYear, aMonth, aDay: Integer;
  aDate: TDateTime;
begin
  if FShortMonthNames.Count < 12 then Exit;
  i := ExpressionCount-1;

  while i > 0 do
  begin
    if (i >= 2) and (Expressions[i].FRecognizedType = etInteger) then
      if Length(Expressions[i].FRecognizedInteger) = 4 then            // Can use StrToInt() ?
        if (StrToInt(Expressions[i].FRecognizedInteger) > 1940) and (StrToInt(Expressions[i].FRecognizedInteger) < 2100) then         // We are in presence of a year!?
        begin
          // Expressions can be separed with large spacing
          pxTolerence := GetPagePixelsWidth(Expressions[i].FPageNumber) div 5;  // 1/5 sheet width

          if ExpressionsSideBySide(i-1, i, pxTolerence) then
          begin
            aMonth := RecognizeMonth(Expressions[i-1].FDERValue);

            if aMonth <> 0 then
              if ExpressionsSideBySide(i-2, i-1, pxTolerence) then
                if Expressions[i-2].FRecognizedType = etInteger then
                  if length(Expressions[i-2].FRecognizedInteger) < 3 then // We are in presence of a day !?
                    try
                      // Validate date :
                      aYear := StrToInt(Expressions[i].FRecognizedInteger);
                      aDay  := StrToInt(Expressions[i-2].FRecognizedInteger);

                      if (aDay > 0) and (aDay < 32) then
                        if TryToEncodeDate(aYear, aMonth, aDay, aDate) then
                        begin
                          // Merge 3 expressions into a date :
                          MergeExpressions(i, i-1);
                          Dec(i);
                          MergeExpressions(i, i-1);  // Carefull: ExpressionCount was modified by MergeExpressions() call ...
                          Dec(i);

                          with Expressions[i] do
                          begin
                            FRecognizedDate := intToStr(Trunc(aDate));
                            FRecognizedType := etDate;
                          end;
                        end;
                    except

                    end;
          end;
        end;

    Dec(i);
  end;
end;

function TcyDocER.GetOCRTextFromRect(const FromRect: TRect; const FromPage: Integer; const AllowPartiallyInside: Boolean = true): String;
var
  i: Integer;
  IgnoreFromRect: Boolean;
  IncludeExpression: Boolean;
  StrConexion: String;
begin
  Result := '';
  IgnoreFromRect := (FromRect.Left = 0) and (FromRect.Top = 0) and (FromRect.Right = 0) and (FromRect.Bottom = 0);

  for i := 0 to ExpressionCount-1 do
  begin
    IncludeExpression := (FromPage = 0) or (FromPage = Expressions[i].PageNumber);

    if IncludeExpression and (not IgnoreFromRect) then
      IncludeExpression := ExpressionInRect(i, FromRect, AllowPartiallyInside);

    if IncludeExpression then
    begin
      StrConexion := '';

      if (Result <> '') and (i > 0) then
        if Expressions[i].FPageNumber = Expressions[i-1].FPageNumber then
          if ExpressionsInSameLine(i, i-1)
          then StrConexion := ' '
          else StrConexion := #13#10;

      Result := Result + StrConexion + Expressions[i].FValue;
    end;
  end;
end;

function TcyDocER.GetOCRText(const FromPage: Integer): String;
begin
  Result := GetOCRTextFromRect(classes.Rect(0, 0, 0, 0), FromPage, true);
end;

function TcyDocER.GetAsDocumentOCRText(const FromPage: Integer = 0): String;
begin
  Result := GetAsDocumentOCRTextFromRect(classes.Rect(0, 0, 0, 0), FromPage, true);
end;

function TcyDocER.GetAsDocumentOCRTextFromRect(const FromRect: TRect; const FromPage: Integer; const AllowPartiallyInside: Boolean = true): String;
var
  i, c, CarsCount, EntersCount: Integer;
  CurrentPage, CurrentPageLinesCount: Integer;
  IgnoreFromRect: Boolean;
  IncludeExpression: Boolean;
  StrConexion: String;

  TotalCars: Int64;
  TotalCarsWidth, TotalExpressionsHeight: Int64;

const
  FontSize = 10;

      function CalcSpaceChars(const _Resolution, _PixelsWidth: Integer): Integer;
      begin
        Result := 0;
        if _Resolution = 0 then Exit;

        Result := round(_PixelsWidth / _Resolution * FontSize);
      end;

      function CalcEnters(const _Resolution, _PixelsHeight: Integer): Integer;
      begin
        Result := 0;
        if _Resolution = 0 then Exit;

        Result := round( (_PixelsHeight / _Resolution * FontSize) / AutoMergeExpressionsRatio);
      end;

      function GetLastLineLength(const aStr: String): Integer;
      var i: Integer;
      begin
        Result := 0;
        for i := Length(aStr) downto 1 do
          if aStr[i] <> #10
          then Inc(Result)
          else Break;
      end;

begin
  // Initialisation :
  Result := '';
  CurrentPage := -1;
  CurrentPageLinesCount := 1;

  // Determine font size :
  TotalCars := 0;
  TotalCarsWidth := 0;
  TotalExpressionsHeight := 0;
  for i := 0 to ExpressionCount-1 do
  begin
    inc(TotalCars, length(Expressions[i].FValue));
    inc(TotalCarsWidth, (Expressions[i].FRightPxPos - Expressions[i].FLeftPxPos));
    inc(TotalExpressionsHeight, (Expressions[i].FBottomPxPos - Expressions[i].FTopPxPos));
  end;

  // Extract text :
  IgnoreFromRect := (FromRect.Left = 0) and (FromRect.Top = 0) and (FromRect.Right = 0) and (FromRect.Bottom = 0);

  for i := 0 to ExpressionCount-1 do
  begin
    if CurrentPage <> Expressions[i].PageNumber then
    begin
      CurrentPage := Expressions[i].PageNumber;
      CurrentPageLinesCount := 1;
    end;

    IncludeExpression := (FromPage = 0) or (FromPage = Expressions[i].PageNumber);

    if IncludeExpression and (not IgnoreFromRect) then
      IncludeExpression := ExpressionInRect(i, FromRect, AllowPartiallyInside);

    if IncludeExpression then
    begin
      if i > 0 then
      begin
        if ExpressionsInSameLine(i, i-1) and (Expressions[i].FPageNumber = Expressions[i-1].FPageNumber) then
        begin
          // *** Continue line *** //
          StrConexion := '';

          // Insert space chars :
          CarsCount := CalcSpaceChars(GetPageResolution(Expressions[i].FPageNumber), Expressions[i].FLeftPxPos) - GetLastLineLength(Result);
          if CarsCount <= 0 then CarsCount := 1;     // At least one char ...

          for c := 1 to CarsCount do
            StrConexion := StrConexion + ' ';
        end
        else begin
          // *** From new line/new page *** //
          StrConexion := '';

          // Insert Enters :
          EntersCount := CalcEnters(GetPageResolution(Expressions[i].FPageNumber), Expressions[i].FTopPxPos) - CurrentPageLinesCount;
          if EntersCount <= 0 then EntersCount := 1; // At least one Enter ...
          for c := 1 to EntersCount do
            StrConexion := StrConexion + #13#10;

          Inc(CurrentPageLinesCount, EntersCount);

          // Insert space chars :
          CarsCount := CalcSpaceChars(GetPageResolution(Expressions[i].FPageNumber), Expressions[i].FLeftPxPos);
          for c := 1 to CarsCount do
            StrConexion := StrConexion + ' ';
        end;
      end
      else begin
        // *** First document line *** //
        StrConexion := '';

        // Insert Enters :
        EntersCount := CalcEnters(GetPageResolution(Expressions[i].FPageNumber), Expressions[i].FTopPxPos);

        for c := 1 to EntersCount do
          StrConexion := StrConexion + #13#10;

        Inc(CurrentPageLinesCount, EntersCount);

        // Insert space chars :
        CarsCount := CalcSpaceChars(GetPageResolution(Expressions[i].FPageNumber), Expressions[i].FLeftPxPos);
        for c := 1 to CarsCount do
          StrConexion := StrConexion + ' ';
      end;

      Result := Result + StrConexion + Expressions[i].FValue;
    end;
  end;
end;

procedure TcyDocER.RotatePageExpressions(const PageNumber, PageWidthBeforeRotation, PageHeightBeforeRotation: Integer; const ToRight: Boolean);
var
  i, SaveLeftPxPos, SaveRightPxPos, SaveTopPxPos, SaveBottomPxPos: Integer;
begin
  for i := 0 to ExpressionCount-1 do
    if (PageNumber = 0) or (Expressions[i].FPageNumber = PageNumber) then
    begin
      SaveLeftPxPos := Expressions[i].FLeftPxPos;
      SaveRightPxPos := Expressions[i].FRightPxPos;
      SaveTopPxPos := Expressions[i].FTopPxPos;
      SaveBottomPxPos := Expressions[i].FBottomPxPos;

      if ToRight then
      begin
        Expressions[i].FLeftPxPos   := PageHeightBeforeRotation - SaveBottomPxPos;
        Expressions[i].FRightPxPos  := PageHeightBeforeRotation - SaveTopPxPos;
        Expressions[i].FTopPxPos    := SaveLeftPxPos;
        Expressions[i].FBottomPxPos := SaveRightPxPos;
      end
      else begin
        Expressions[i].FLeftPxPos   := SaveTopPxPos;
        Expressions[i].FRightPxPos  := SaveBottomPxPos;
        Expressions[i].FTopPxPos    := PageWidthBeforeRotation - SaveRightPxPos;
        Expressions[i].FBottomPxPos := PageWidthBeforeRotation - SaveLeftPxPos;
      end;
    end;
end;

function TcyDocER.IsExpressionsSameValue(const ExpressionIndex1, ExpressionIndex2: Integer): Boolean;
begin
  Result := false;

  if Expressions[ExpressionIndex1].RecognizedType = Expressions[ExpressionIndex2].RecognizedType then
    if Expressions[ExpressionIndex1].RecognizedValue = Expressions[ExpressionIndex2].RecognizedValue then
      Result := true;
end;

function TcyDocER.GetPagePixelsWidth(const PageNumber: Integer; const FromBeginning: Boolean = true): Integer;
begin
  Result := 2480;

  if (PageNumber > 0) and (PageNumber <= PageCount) then
  begin
    if FromBeginning
    then Result := Pages[PageNumber - 1].FPixelsWidth
    else Result := Pages[((PageCount - PageNumber) + 1) - 1].FPixelsWidth;
  end
  else
    if PageCount <> 0 then
      Result := Pages[0].FPixelsWidth;
end;

function TcyDocER.GetPagePixelsHeight(const PageNumber: Integer; const FromBeginning: Boolean = true): Integer;
begin
  Result := 3508;

  if (PageNumber > 0) and (PageNumber <= PageCount) then
  begin
    if FromBeginning
    then Result := Pages[PageNumber - 1].FPixelsHeight
    else Result := Pages[((PageCount - PageNumber) + 1) - 1].FPixelsHeight;
  end
  else
    if PageCount <> 0 then
      Result := Pages[0].FPixelsHeight;
end;

function TcyDocER.GetPageResolution(const PageNumber: Integer; const FromBeginning: Boolean = true): Integer;
begin
  Result := 300;

  if (PageNumber > 0) and (PageNumber <= PageCount) then
  begin
    if FromBeginning
    then Result := Pages[PageNumber - 1].FResolution
    else Result := Pages[((PageCount - PageNumber) + 1) - 1].FResolution;
  end
  else
    if PageCount <> 0 then
      Result := Pages[0].FResolution;
end;

function TcyDocER.MmToPx(const MmValue: Extended; const Resolution: Integer): Integer;
var
  Inches: Extended;
begin
  Inches := MmValue / 25.4;
  Result := Round(Inches * Resolution);
end;

function TcyDocER.MmToPxFromPage(const MmValue: Extended; const PageNumber: Integer): Integer;
begin
  Result := MmToPx(MmValue, GetPageResolution(PageNumber));
end;

function TcyDocER.PxToMmFromPage(const pxValue: Extended; const PageNumber: Integer): Extended;
begin
  Result := PxToMm(pxValue, GetPageResolution(PageNumber));
end;

function TcyDocER.PxToMm(const pxValue: Extended; const Resolution: Integer): Extended;
var
  Inches: Extended;
begin
  if Resolution = 0 then
  begin
    Result := 0;
    Exit;
  end;

  Inches := pxValue / Resolution;
  Result := Inches * 25.4;
end;

end.
